import unittest

from robot.output.listeners import Listeners, LibraryListeners
from robot.output import LOGGER
from robot.running.outputcapture import OutputCapturer
from robot.utils.asserts import assert_equal, assert_raises
from robot.utils import DotDict


LOGGER.unregister_console_logger()


class Mock:
    non_existing = ()

    def __getattr__(self, name):
        if name[:2] == '__' or name in self.non_existing:
            raise AttributeError
        return ''


class SuiteMock(Mock):

    def __init__(self):
        self.name = 'suitemock'
        self.doc = 'somedoc'
        self.status = 'PASS'
        self.tests = self.suites = []

    stat_message = 'stat message'
    full_message = 'full message'


class TestMock(Mock):

    def __init__(self):
        self.name = 'testmock'
        self.doc = 'cod'
        self.tags = ['foo', 'bar']
        self.message = 'Expected failure'
        self.status = 'FAIL'
        self.data = DotDict({'name':self.name})


class KwMock(Mock):
    non_existing = ('branch_status',)

    def __init__(self):
        self.name = 'kwmock'
        self.args = ['a1', 'a2']
        self.status = 'PASS'
        self.type = 'kw'


class ListenOutputs:

    def output_file(self, path):
        self._out_file('Output', path)

    def report_file(self, path):
        self._out_file('Report', path)

    def log_file(self, path):
        self._out_file('Log', path)

    def debug_file(self, path):
        self._out_file('Debug', path)

    def xunit_file(self, path):
        self._out_file('XUnit', path)

    def _out_file(self, name, path):
        print('%s: %s' % (name, path))


class ListenAll(ListenOutputs):
    ROBOT_LISTENER_API_VERSION = '2'

    def start_suite(self, name, attrs):
        print("SUITE START: %s '%s'" % (name, attrs['doc']))

    def start_test(self, name, attrs):
        print("TEST START: %s '%s' %s" % (name, attrs['doc'],
                                          ', '.join(attrs['tags'])))

    def start_keyword(self, name, attrs):
        args = [str(arg) for arg in attrs['args']]
        print("KW START: %s %s" % (name, args))

    def end_keyword(self, name, attrs):
        print("KW END: %s" % attrs['status'])

    def end_test(self, name, attrs):
        if attrs['status'] == 'PASS':
            print('TEST END: PASS')
        else:
            print("TEST END: %s %s" % (attrs['status'], attrs['message']))

    def end_suite(self, name, attrs):
        print('SUITE END: %s %s' % (attrs['status'], attrs['statistics']))

    def close(self):
        print('Closing...')


class TestListeners(unittest.TestCase):
    listener_name = 'test_listeners.ListenAll'
    stat_message = 'stat message'

    def setUp(self):
        self.listeners = Listeners([self.listener_name])
        self.capturer = OutputCapturer()

    def test_start_suite(self):
        self.listeners.start_suite(SuiteMock())
        self._assert_output("SUITE START: suitemock 'somedoc'")

    def test_start_test(self):
        self.listeners.start_test(TestMock())
        self._assert_output("TEST START: testmock 'cod' foo, bar")

    def test_start_keyword(self):
        self.listeners.start_keyword(KwMock())
        self._assert_output("KW START: kwmock ['a1', 'a2']")

    def test_end_keyword(self):
        self.listeners.end_keyword(KwMock())
        self._assert_output("KW END: PASS")

    def test_end_test(self):
        self.listeners.end_test(TestMock())
        self._assert_output('TEST END: FAIL Expected failure')

    def test_end_suite(self):
        self.listeners.end_suite(SuiteMock())
        self._assert_output('SUITE END: PASS ' + self.stat_message)

    def test_output_file(self):
        self.listeners.output_file('output', 'path/to/output')
        self._assert_output('Output: path/to/output')

    def test_log_file(self):
        self.listeners.output_file('log', 'path/to/log')
        self._assert_output('Log: path/to/log')

    def test_report_file(self):
        self.listeners.output_file('report', 'path/to/report')
        self._assert_output('Report: path/to/report')

    def test_debug_file(self):
        self.listeners.output_file('debug', 'path/to/debug')
        self._assert_output('Debug: path/to/debug')

    def test_xunit_file(self):
        self.listeners.output_file('XUnit', 'path/to/xunit')
        self._assert_output('XUnit: path/to/xunit')

    def test_close(self):
        self.listeners.close()
        self._assert_output('Closing...')

    def _assert_output(self, expected):
        stdout, stderr = self.capturer._release()
        assert_equal(stderr, '')
        assert_equal(stdout.rstrip(), expected)


class TestAttributesAreNotAccessedUnnecessarily(unittest.TestCase):

    def test_start_and_end_methods(self):
        class ModelStub:
            IF_ELSE_ROOT = 'IF/ELSE ROOT'
            TRY_EXCEPT_ROOT = 'TRY/EXCEPT ROOT'
            type = 'xxx'
        for listeners in [Listeners([]), LibraryListeners()]:
            for name in dir(listeners):
                if name.startswith(('start_', 'end_')):
                    model = ModelStub() if name.endswith('keyword') else None
                    method = getattr(listeners, name)
                    method(model)

    def test_message_methods(self):
        class Message:
            level = 'INFO'
        for listeners in [Listeners([]), LibraryListeners()]:
            listeners.log_message(Message)
            listeners.message(Message)

    def test_some_methods_implemented(self):
        class MyListener:
            ROBOT_LISTENER_API_VERSION = 2
            def end_suite(self, suite):
                pass
        libs = LibraryListeners()
        libs.new_suite_scope()
        libs.register([MyListener()], None)
        for listeners in [Listeners([MyListener()]), libs]:
            listeners.start_suite(None)
            assert_raises(AttributeError, listeners.end_suite, None)


if __name__ == '__main__':
    unittest.main()



ROBOT_LISTENER_API_VERSION = 2


class ListenerMethod:

    def __init__(self, name):
        self.__name__ = name
        self.failed = False

    def __call__(self, *args, **kws):
        if not self.failed:
            self.failed = True
            raise AssertionError("Expected failure in %s!" % self.__name__)


for name in ['start_suite', 'end_suite', 'start_test', 'end_test',
             'start_keyword', 'end_keyword', 'log_message', 'message',
             'output_file', 'log_file', 'report_file', 'debug_file', 'close']:
    globals()[name] = ListenerMethod(name)



from robot.api import logger
from robot.api.deco import keyword
from robot.libraries.BuiltIn import BuiltIn


should_be_equal = BuiltIn().should_be_equal
log = logger.write


@keyword(name="User ${user} Selects ${item} From Webshop")
def user_selects_from_webshop(user, item):
    log("This is always executed")
    return user, item


@keyword(name='${prefix:Given|When|Then} this "${item}" ${no good name for this arg ...}')
def this(ignored_prefix, item, somearg):
    log("%s-%s" % (item, somearg))


@keyword(name="My embedded ${var}")
def my_embedded(var):
    should_be_equal(var, "warrior")


@keyword(name=r"${x:x} gets ${y:\w} from the ${z:.}")
def gets_from_the(x, y, z):
    should_be_equal("%s-%s-%s" % (x, y, z), "x-y-z")


@keyword(name="${a}-lib-${b}")
def mult_match1(a, b):
    log("%s-lib-%s" % (a, b))


@keyword(name="${a}+lib+${b}")
def mult_match2(a, b):
    log("%s+lib+%s" % (a, b))


@keyword(name="${a}*lib*${b}")
def mult_match3(a, b):
    log("%s*lib*%s" % (a, b))


@keyword(name='I execute "${x:[^"]*}"')
def i_execute(x):
    should_be_equal(x, "foo")


@keyword(name='I execute "${x:bar}" with "${y:...}"')
def i_execute_with(x, y):
    should_be_equal(x, "bar")
    should_be_equal(y, "zap")


@keyword(name=r"Result of ${a:\d+} ${operator:[+-]} ${b:\d+} is ${result}")
def result_of_is(a, operator, b, result):
    should_be_equal(eval("%s%s%s" % (a, operator, b)), float(result))


@keyword(name="I want ${integer:whatever} and ${string:everwhat} as variables")
def i_want_as_variables(integer, string):
    should_be_equal(integer, 42)
    should_be_equal(string, "42")


@keyword(name=r"Today is ${date:\d{4}-\d{2}-\d{2}}")
def today_is(date):
    should_be_equal(date, "2011-06-21")


@keyword(name=r"Today is ${day1:\w{6,9}} and tomorrow is ${day2:\w{6,9}}")
def today_is_and_tomorrow_is(day1, day2):
    should_be_equal(day1, "Tuesday")
    should_be_equal(day2, "Wednesday")


@keyword(name=r"Literal ${Curly:\{} Brace")
def literal_opening_curly_brace(curly):
    should_be_equal(curly, "{")


@keyword(name="Literal ${Curly:\}} Brace")
def literal_closing_curly_brace(curly):
    should_be_equal(curly, "}")


@keyword(name="Literal ${Curly:{}} Braces")
def literal_curly_braces(curly):
    should_be_equal(curly, "{}")


@keyword(name=r"Custom Regexp With Escape Chars e.g. ${1E:\\}, "
              r"${2E:\\\\} and ${PATH:c:\\temp\\.*}")
def custom_regexp_with_escape_chars(e1, e2, path):
    should_be_equal(e1, "\\")
    should_be_equal(e2, "\\\\")
    should_be_equal(path, "c:\\temp\\test.txt")


@keyword(name=r"Custom Regexp With ${escapes:\\\}}")
def custom_regexp_with_escapes_1(escapes):
    should_be_equal(escapes, r'\}')


@keyword(name=r"Custom Regexp With ${escapes:\\\{}")
def custom_regexp_with_escapes_2(escapes):
    should_be_equal(escapes, r'\{')


@keyword(name=r"Custom Regexp With ${escapes:\\{}}")
def custom_regexp_with_escapes_3(escapes):
    should_be_equal(escapes, r'\{}')


@keyword(name=r"Grouping ${x:Cu(st|ts)(om)?} ${y:Regexp\(?erts\)?}")
def grouping(x, y):
    return f'{x}-{y}'


@keyword(name="Wrong ${number} of embedded ${args}")
def too_few_args_here(arg):
    pass


@keyword(name="Optional ${nonembedded} Args Are ${okay}")
def optional_args_are_okay(nonembedded=1, okay=2, indeed=3):
    pass


@keyword(name="Varargs With ${embedded} Args Are ${okay}")
def varargs_are_okay(*args):
    return args


@keyword('It is ${vehicle:a (car|ship)}')
def same_name_1(vehicle):
    log(vehicle)


@keyword('It is ${animal:a (dog|cat)}')
def same_name_2(animal):
    log(animal)


@keyword('It is ${animal:a (cat|cow)}')
def same_name_3(animal):
    log(animal)


@keyword('It is totally ${same}')
def totally_same_1(arg):
    raise Exception('Not executed')


@keyword('It is totally ${same}')
def totally_same_2(arg):
    raise Exception('Not executed')



from functools import partialmethod


class PartialMethod:

    def method(self, value, expected, lower: bool = False):
        if lower is True:
            value = value.lower()
        assert value == expected

    partial_method = partialmethod(method, expected='value')



import unittest

from robot.model.metadata import Metadata
from robot.utils.asserts import assert_equal


class TestMetadata(unittest.TestCase):

    def test_normalization(self):
        md = Metadata([('m1', 'xxx'), ('M2', 'xxx'), ('m_3', 'xxx'),
                       ('M1', 'YYY'), ('M 3', 'YYY')])
        assert_equal(dict(md), {'m1': 'YYY', 'M2': 'xxx', 'm_3': 'YYY'})

    def test_str(self):
        assert_equal(str(Metadata()), '{}')
        d = {'a': 1, 'B': 'two', 'ä': 'neljä'}
        assert_equal(str(Metadata(d)), '{a: 1, B: two, ä: neljä}')

    def test_non_string_items(self):
        md = Metadata([('number', 42), ('boolean', True), (1, 'one')])
        assert_equal(md['number'], '42')
        assert_equal(md['boolean'], 'True')
        assert_equal(md['1'], 'one')
        md['number'] = 1.0
        md['boolean'] = False
        md['new'] = []
        md[True] = ''
        assert_equal(md['number'], '1.0')
        assert_equal(md['boolean'], 'False')
        assert_equal(md['new'], '[]')
        assert_equal(md['True'], '')
        md.setdefault('number', 99)
        md.setdefault('setdefault', 99)
        assert_equal(md['number'], '1.0')
        assert_equal(md['setdefault'], '99')


if __name__ == '__main__':
    unittest.main()



import os


class BinaryDataLibrary:

    def print_bytes(self):
        """Prints all bytes in range 0-255. Many of them are control chars."""
        for i in range(256):
            print("*INFO* Byte %d: '%s'" % (i, chr(i)))
        print("*INFO* All bytes printed successfully")

    def raise_byte_error(self):
        raise AssertionError("Bytes 0, 10, 127, 255: '%s', '%s', '%s', '%s'"
                             % (chr(0), chr(10), chr(127), chr(255)))

    def print_binary_data(self):
        print(os.urandom(100))
        print("*INFO* Binary data printed successfully")



from listenerlibrary import listenerlibrary


class multiple_listenerlibrary:

    def __init__(self, fail=False):
        self.instances = [
            listenerlibrary(),
            listenerlibrary(),
        ]
        if fail:
            class NoVersionListener:
                def events_should_be_empty(self):
                    pass
            self.instances.append(NoVersionListener())
        self.ROBOT_LIBRARY_LISTENER = self.instances

    def events_should_be(self, *expected):
        for inst in self.instances:
            inst.events_should_be(*expected)

    def events_should_be_empty(self):
        for inst in self.instances:
            inst.events_should_be_empty()



KEYWORDS = [('other than strings', [1, 2]),
            ('named args before positional', ['a=1', 'b']),
            ('multiple varargs', ['*first', '*second']),
            ('kwargs before positional args', ['**kwargs', 'a']),
            ('kwargs before named args', ['**kwargs', 'a=1']),
            ('kwargs before varargs', ['**kwargs', '*varargs']),
            ('empty tuple', ['arg', ()]),
            ('too long tuple', [('too', 'long', 'tuple')]),
            ('too long tuple with *varargs', [('*too', 'long')]),
            ('too long tuple with **kwargs', [('**too', 'long')]),
            ('tuple with non-string first value', [(None,)]),
            ('valid argspec', ['a']),
            ('valid argspec with tuple', [['a'], ('b', None)])]


class InvalidArgSpecs:

    def get_keyword_names(self):
        return [name for name, _ in KEYWORDS]

    def run_keyword(self, name, args, kwargs):
        return ' '.join(args + tuple(kwargs)).upper()

    def get_keyword_arguments(self, name):
        return dict(KEYWORDS)[name]



class ArgumentsPython:

    # method docs are used in unit tests as expected min and max args

    def a_0(self):
        """(0,0)"""
        return 'a_0'

    def a_1(self, arg):
        """(1,1)"""
        return 'a_1: ' + arg

    def a_3(self, arg1, arg2, arg3):
        """(3,3)"""
        return ' '.join(['a_3:',arg1,arg2,arg3])

    def a_0_1(self, arg='default'):
        """(0,1)"""
        return 'a_0_1: ' + arg

    def a_1_3(self, arg1, arg2='default', arg3='default'):
        """(1,3)"""
        return ' '.join(['a_1_3:',arg1,arg2,arg3])

    def a_0_n(self, *args):
        """(0,sys.maxsize)"""
        return ' '.join(['a_0_n:', ' '.join(args)])

    def a_1_n(self, arg, *args):
        """(1,sys.maxsize)"""
        return ' '.join(['a_1_n:', arg, ' '.join(args)])

    def a_1_2_n(self, arg1, arg2='default', *args):
        """(1,sys.maxsize)"""
        return ' '.join(['a_1_2_n:', arg1, arg2, ' '.join(args)])



ROBOT_LISTENER_API_VERSION = 2


from robot.libraries.BuiltIn import BuiltIn

run_keyword = BuiltIn().run_keyword


def start_suite(name, attrs):
    run_keyword('Log', 'start_suite')


def end_suite(name, attrs):
    run_keyword('Log', 'end_suite')


def start_test(name, attrs):
    run_keyword('Log', 'start_test')


def end_test(name, attrs):
    run_keyword('Log', 'end_test')


def start_keyword(name, attrs):
    run_keyword('Log', 'start_keyword')


def end_keyword(name, attrs):
    run_keyword('Log', 'end_keyword')



from datetime import date
from decimal import Decimal

from robot.api.deco import keyword


@keyword(types=[int, Decimal, bool, date, list])
def basics(integer, decimal, boolean, date_, list_=None):
    _validate_type(integer, 42)
    _validate_type(decimal, Decimal('3.14'))
    _validate_type(boolean, True)
    _validate_type(date_, date(2018, 8, 30))
    _validate_type(list_, ['foo'])


@keyword(types=[int, None, float])
def none_means_no_type(foo, bar, zap):
    _validate_type(foo, 1)
    _validate_type(bar, '2')
    _validate_type(zap, 3.0)


@keyword(types=['', int, False])
def falsy_types_mean_no_type(foo, bar, zap):
    _validate_type(foo, '1')
    _validate_type(bar, 2)
    _validate_type(zap, '3')


@keyword(types=[int, type(None), float])
def nonetype(foo, bar, zap):
    _validate_type(foo, 1)
    _validate_type(bar, None)
    _validate_type(zap, 3.0)


@keyword(types=[int, 'None', float])
def none_as_string_is_none(foo, bar, zap):
    _validate_type(foo, 1)
    _validate_type(bar, None)
    _validate_type(zap, 3.0)


@keyword(types=[(int, None), (None,)])
def none_in_tuple_is_alias_for_nonetype(arg1, arg2, exp1=None, exp2=None):
    _validate_type(arg1, eval(exp1) if exp1 else None)
    _validate_type(arg2, eval(exp2) if exp2 else None)


@keyword(types=[int, float])
def less_types_than_arguments_is_ok(foo, bar, zap):
    _validate_type(foo, 1)
    _validate_type(bar, 2.0)
    _validate_type(zap, '3')


@keyword(types=[int, int])
def too_many_types(argument):
    raise RuntimeError('Should not be executed!')


@keyword(types=[int, int, int])
def varargs_and_kwargs(arg, *varargs, **kwargs):
    _validate_type(arg, 1)
    _validate_type(varargs, (2, 3, 4))
    _validate_type(kwargs, {'kw': 5})


@keyword(types=[None, int, float])
def kwonly(*, foo, bar=None, zap):
    _validate_type(foo, '1')
    _validate_type(bar, 2)
    _validate_type(zap, 3.0)


@keyword(types=[None, None, int, float, Decimal])
def kwonly_with_varargs_and_kwargs(*varargs, foo, bar=None, zap, **kwargs):
    _validate_type(varargs, ('0',))
    _validate_type(foo, '1')
    _validate_type(bar, 2)
    _validate_type(zap, 3.0)
    _validate_type(kwargs, {'quux': Decimal(4)})


def _validate_type(argument, expected):
    if argument != expected or type(argument) != type(expected):
        raise AssertionError('%r (%s) != %r (%s)'
                             % (argument, type(argument).__name__,
                                expected, type(expected).__name__))



def normal_args(a, b=None):
    pass


def normal_and_varargs(a, b=None, *varargs):
    pass


def normal_and_varargs_and_kwargs(a, b=None, *varargs, **kwargs):
    pass


def normal_and_kwargs(positional, **kwargs):
    pass


def varargs_and_kwargs(*varargs, **kwargs):
    pass


def kwargs(**kwargs):
    pass



from robot.api.deco import keyword


@keyword('Embedded "${argument}" in library')
def embedded(arg):
    print(arg)


@keyword('Embedded object "${obj}" in library')
def embedded_object(obj):
    print(obj)
    if obj.name != 'Robot':
        raise AssertionError(f"'{obj.name}' != 'Robot'")



from robot.api.deco import keyword


class ClassWithAutoKeywordsOff:
    ROBOT_AUTO_KEYWORDS = False

    def public_method_is_not_keyword(self):
        raise RuntimeError('Should not be executed!')

    @keyword(name="Decorated Method Is Keyword")
    def decorated_method(self):
        print('Decorated methods are keywords.')

    def _private_method_is_not_keyword(self):
        raise RuntimeError('Should not be executed!')

    @keyword
    def _private_decorated_method_is_keyword(self):
        print('Decorated private methods are keywords.')



from os.path import abspath, dirname, join, normpath
import unittest
from pathlib import Path

from robot.conf.settings import _BaseSettings, RobotSettings, RebotSettings
from robot.errors import DataError
from robot.utils import WINDOWS
from robot.utils.asserts import assert_equal, assert_true


class SettingWrapper(_BaseSettings):

    def __init__(self):
        pass


class TestRobotAndRebotSettings(unittest.TestCase):

    def test_robot_and_rebot_settings_are_independent_1(self):
        # https://github.com/robotframework/robotframework/issues/881
        orig_opts = RobotSettings()._opts
        RebotSettings()
        assert_equal(RobotSettings()._opts, orig_opts)

    def test_robot_and_rebot_settings_are_independent_2(self):
        # https://github.com/robotframework/robotframework/pull/2438
        rebot = RebotSettings()
        assert_equal(rebot['TestNames'], [])
        robot = RobotSettings()
        robot['TestNames'].extend(['test1', 'test2'])
        assert_equal(rebot['TestNames'], [])

    def test_robot_settings_are_independent(self):
        settings1 = RobotSettings()
        assert_equal(settings1['Include'], [])
        settings2 = RobotSettings()
        settings2['Include'].append('tag')
        assert_equal(settings1['Include'], [])

    def test_extra_options(self):
        assert_equal(RobotSettings(name='My Name')['Name'], 'My Name')
        assert_equal(RobotSettings({'name': 'Override'}, name='Set')['Name'],'Set')

    def test_multi_options_as_single_string(self):
        assert_equal(RobotSettings({'test': 'one'})['TestNames'], ['one'])
        assert_equal(RebotSettings({'exclude': 'two'})['Exclude'], ['two'])

    def test_output_files_as_none_string(self):
        for name in 'Output', 'Report', 'Log', 'XUnit', 'DebugFile':
            attr = (name[:-4] if name.endswith('File') else name).lower()
            settings = RobotSettings({name.lower(): 'NoNe'})
            assert_equal(settings[name], None)
            if hasattr(settings, attr):
                assert_equal(getattr(settings, attr), None)

    def test_output_files_as_none_object(self):
        for name in 'Output', 'Report', 'Log', 'XUnit', 'DebugFile':
            attr = (name[:-4] if name.endswith('File') else name).lower()
            settings = RobotSettings({name.lower(): None})
            assert_equal(settings[name], None)
            if hasattr(settings, attr):
                assert_equal(getattr(settings, attr), None)

    def test_output_files_as_pathlib_objects(self):
        for name in 'Output.xml', 'Report.html', 'Log.html', 'XUnit.xml', 'DebugFile.txt':
            name, ext = name.split('.')
            expected = abspath(f'test.{ext}')
            attr = (name[:-4] if name.endswith('File') else name).lower()
            settings = RobotSettings({name.lower(): Path('test')})
            assert_equal(settings[name], expected)
            if hasattr(settings, attr):
                assert_equal(getattr(settings, attr), expected)

    def test_output_dir_as_pathlib_object(self):
        assert_equal(RobotSettings({'outputdir': Path('.')})['OutputDir'], abspath('.'))

    def test_rerun_failed_as_none_string_and_object(self):
        for name in 'ReRunFailed', 'ReRunFailedSuites':
            assert_equal(RobotSettings({name.lower(): 'NONE'})[name], None)
            assert_equal(RobotSettings({name.lower(): 'NoNe'})[name], None)
            assert_equal(RobotSettings({name.lower(): None})[name], None)

    def test_rerun_failed_as_pathlib_object(self):
        for name in 'ReRunFailed', 'ReRunFailedSuites':
            assert_equal(RobotSettings({name.lower(): Path('R.xml')})[name], 'R.xml')

    def test_doc(self):
        assert_equal(RobotSettings()['Doc'], None)
        assert_equal(RobotSettings({'doc': None})['Doc'], None)
        assert_equal(RobotSettings({'doc': 'The doc!'})['Doc'], 'The doc!')

    def test_doc_from_file(self):
        for doc in __file__, Path(__file__):
            doc = RobotSettings({'doc': doc})['Doc']
            assert_true('def test_doc_from_file(self):' in doc)

    def test_log_levels(self):
        self._verify_log_level('TRACE')
        self._verify_log_level('DEBUG')
        self._verify_log_level('INFO')
        self._verify_log_level('WARN')
        self._verify_log_level('NONE')

    def test_default_log_level(self):
        self._verify_log_levels(RobotSettings(), 'INFO')
        self._verify_log_levels(RebotSettings(), 'TRACE')

    def test_pythonpath(self):
        curdir = normpath(dirname(abspath(__file__)))
        for inp, exp in [('foo', [abspath('foo')]),
                         (['a:b:c', 'zap'], [abspath(p) for p in ('a', 'b', 'c', 'zap')]),
                         (['foo;bar', 'zap'], [abspath(p) for p in ('foo', 'bar', 'zap')]),
                         (join(curdir, 't*_set*.??'), [join(curdir, 'test_settings.py')])]:
            assert_equal(RobotSettings(pythonpath=inp).pythonpath, exp)
        if WINDOWS:
            assert_equal(RobotSettings(pythonpath=r'c:\temp:d:\e\f:g').pythonpath,
                         [r'c:\temp', r'd:\e\f', abspath('g')])
            assert_equal(RobotSettings(pythonpath=r'c:\temp;d:\e\f;g').pythonpath,
                         [r'c:\temp', r'd:\e\f', abspath('g')])

    def test_get_rebot_settings_returns_only_rebot_settings(self):
        expected = RebotSettings()
        for opt in RobotSettings().get_rebot_settings()._opts:
            assert_true(opt in expected, opt)

    def test_get_rebot_settings_excludes_settings_handled_already_in_execution(self):
        settings = RobotSettings(
            name='N', doc=':doc:', metadata='m:d', settag='s',
            include='i', exclude='e', test='t', suite='s',
            output='out.xml', loglevel='DEBUG:INFO', timestampoutputs=True
        ).get_rebot_settings()
        for name in 'Name', 'Doc', 'Output':
            assert_equal(settings[name], None)
        for name in 'Metadata', 'SetTag', 'Include', 'Exclude', 'TestNames', 'SuiteNames':
            assert_equal(settings[name], [])
        assert_equal(settings['LogLevel'], 'TRACE')
        assert_equal(settings['TimestampOutputs'], False)

    def _verify_log_level(self, input, level=None, default=None):
        level = level or input
        default = default or level
        self._verify_log_levels(RobotSettings({'loglevel': input}), level, default)
        self._verify_log_levels(RebotSettings({'loglevel': input}), level, default)

    def _verify_log_levels(self, settings, level, default=None):
        assert_equal(settings['LogLevel'], level)
        assert_equal(settings['VisibleLogLevel'], default or level)

    def test_log_levels_with_default(self):
        self._verify_log_level('TRACE:INFO', level='TRACE', default='INFO')
        self._verify_log_level('TRACE:debug', level='TRACE', default='DEBUG')
        self._verify_log_level('DEBUG:INFO', level='DEBUG', default='INFO')

    def test_invalid_log_level(self):
        self._verify_invalid_log_level('kekonen')
        self._verify_invalid_log_level('DEBUG:INFO:FOO')
        self._verify_invalid_log_level('INFO:bar')
        self._verify_invalid_log_level('bar:INFO')

    def test_visible_level_higher_than_normal_level(self):
        self._verify_invalid_log_level('INFO:TRACE')
        self._verify_invalid_log_level('DEBUG:TRACE')

    def _verify_invalid_log_level(self, input):
        self.assertRaises(DataError, RobotSettings, {'loglevel': input})


if __name__ == '__main__':
    unittest.main()



from functools import partial


def function(value, expected, lower=False):
    if lower is True:
        value = value.lower()
    assert value == expected


partial_function = partial(function, expected='value')



import unittest
import sys

from robot.utils.asserts import assert_equal, assert_not_none
from robot.utils.encodingsniffer import get_console_encoding
from robot.utils import WINDOWS


class StreamStub:

    def __init__(self, encoding, isatty=True):
        self.encoding = encoding
        self.isatty = lambda: isatty


class TestGetConsoleEncodingFromStandardStreams(unittest.TestCase):

    def setUp(self):
        self._orig_streams = sys.__stdout__, sys.__stderr__, sys.__stdin__

    def tearDown(self):
        sys.__stdout__, sys.__stderr__, sys.__stdin__ = self._orig_streams

    def test_valid_encoding(self):
        sys.__stdout__ = StreamStub('ASCII')
        assert_equal(get_console_encoding(), 'ASCII')

    def test_invalid_encoding(self):
        sys.__stdout__ = StreamStub('invalid')
        sys.__stderr__ = StreamStub('ascII')
        assert_equal(get_console_encoding(), 'ascII')

    def test_no_encoding(self):
        sys.__stdout__ = object()
        sys.__stderr__ = object()
        sys.__stdin__ = StreamStub('ascii')
        assert_equal(get_console_encoding(), 'ascii')
        sys.__stdin__ = object()
        assert_not_none(get_console_encoding())

    def test_none_encoding(self):
        sys.__stdout__ = StreamStub(None)
        sys.__stderr__ = StreamStub(None)
        sys.__stdin__ = StreamStub('ascii')
        assert_equal(get_console_encoding(), 'ascii')
        sys.__stdin__ = StreamStub(None)
        assert_not_none(get_console_encoding())

    def test_non_tty_streams_are_not_used(self):
        sys.__stdout__ = StreamStub('utf-8', isatty=False)
        sys.__stderr__ = StreamStub('latin-1', isatty=False)
        sys.__stdin__ = StreamStub('ascii')
        assert_equal(get_console_encoding(), 'ascii')


# We don't look at streams on Windows. Our `isatty` doesn't consider StreamSub a tty.
if WINDOWS:
    del TestGetConsoleEncodingFromStandardStreams


if __name__ == '__main__':
    unittest.main()



from itertools import chain
import unittest
from robot.utils.asserts import assert_equal, assert_raises_with_msg, assert_true

from robot.errors import DataError
from robot.result import Keyword, TestCase, TestSuite
from robot.result.configurer import SuiteConfigurer


SETUP = Keyword.SETUP
TEARDOWN = Keyword.TEARDOWN


class TestSuiteAttributes(unittest.TestCase):

    def setUp(self):
        self.suite = TestSuite(name='Suite', metadata={'A A': '1', 'bb': '1'})
        self.suite.tests.create(name='Make suite non-empty')

    def test_name_and_doc(self):
        self.suite.visit(SuiteConfigurer(name='New Name', doc='New Doc'))
        assert_equal(self.suite.name, 'New Name')
        assert_equal(self.suite.doc, 'New Doc')

    def test_metadata(self):
        self.suite.visit(SuiteConfigurer(metadata={'bb': '2', 'C': '2'}))
        assert_equal(self.suite.metadata, {'A A': '1', 'bb': '2', 'C': '2'})

    def test_metadata_is_normalized(self):
        self.suite.visit(SuiteConfigurer(metadata={'aa': '2', 'B_B': '2'}))
        assert_equal(self.suite.metadata, {'A A': '2', 'bb': '2'})


class TestTestAttributes(unittest.TestCase):

    def setUp(self):
        self.suite = TestSuite()
        self.suite.tests = [TestCase()]
        self.suite.suites = [TestSuite()]
        self.suite.suites[0].tests = [TestCase(tags=['tag'])]

    def test_set_tags(self):
        self.suite.visit(SuiteConfigurer(set_tags=['new']))
        assert_equal(list(self.suite.tests[0].tags), ['new'])
        assert_equal(list(self.suite.suites[0].tests[0].tags), ['new', 'tag'])

    def test_tags_are_normalized(self):
        self.suite.visit(SuiteConfigurer(set_tags=['TAG', '', 't a g', 'NONE']))
        assert_equal(list(self.suite.tests[0].tags), ['TAG'])
        assert_equal(list(self.suite.suites[0].tests[0].tags), ['tag'])

    def test_remove_negative_tags(self):
        self.suite.visit(SuiteConfigurer(set_tags=['n', '-TAG']))
        assert_equal(list(self.suite.tests[0].tags), ['n'])
        assert_equal(list(self.suite.suites[0].tests[0].tags), ['n'])

    def test_remove_negative_tags_using_pattern(self):
        self.suite.visit(SuiteConfigurer(set_tags=['-t*', '-nomatch']))
        assert_equal(list(self.suite.tests[0].tags), [])
        assert_equal(list(self.suite.suites[0].tests[0].tags), [])


class TestFiltering(unittest.TestCase):

    def setUp(self):
        self.suite = TestSuite(name='root')
        self.suite.tests = [TestCase(name='n0'), TestCase(name='n1', tags=['t1']),
                            TestCase(name='n2', tags=['t1', 't2'])]
        self.suite.suites.create(name='sub').tests.create(name='n1', tags=['t1'])

    def test_include(self):
        self.suite.visit(SuiteConfigurer(include_tags=['t1', 'none', '', '?2']))
        assert_equal([t.name for t in self.suite.tests], ['n1', 'n2'])
        assert_equal([t.name for t in self.suite.suites[0].tests], ['n1'])

    def test_exclude(self):
        self.suite.visit(SuiteConfigurer(exclude_tags=['t1', '?1ANDt2']))
        assert_equal([t.name for t in self.suite.tests], ['n0'])
        assert_equal(list(self.suite.suites), [])

    def test_include_by_names(self):
        self.suite.visit(SuiteConfigurer(include_suites=['s?b', 'xxx'],
                                         include_tests=['', '*1', 'xxx']))
        assert_equal(list(self.suite.tests), [])
        assert_equal([t.name for t in self.suite.suites[0].tests], ['n1'])

    def test_no_matching_tests_with_one_selector_each(self):
        configurer = SuiteConfigurer(include_tags='i', exclude_tags='e',
                                     include_suites='s', include_tests='t')
        assert_raises_with_msg(
            DataError,
            "Suite 'root' contains no tests matching tag 'i', "
            "not matching tag 'e' and matching name 't' in suite 's'.",
            self.suite.visit, configurer
        )

    def test_no_matching_tests_with_multiple_selectors(self):
        configurer = SuiteConfigurer(include_tags=['i1', 'i2'],
                                     exclude_tags=['e1', 'e2'],
                                     include_suites=['s1', 's2', 's3'],
                                     include_tests=['t1', 't2'])
        assert_raises_with_msg(
            DataError,
            "Suite 'root' contains no tests matching tags 'i1' or 'i2', "
            "not matching tags 'e1' or 'e2' and matching name 't1' or 't2' "
            "in suites 's1', 's2' or 's3'.",
            self.suite.visit, configurer
        )

    def test_empty_suite(self):
        suite = TestSuite(name='x')
        suite.visit(SuiteConfigurer(empty_suite_ok=True))
        assert_raises_with_msg(DataError,
                               "Suite 'x' contains no tests.",
                               suite.visit, SuiteConfigurer())


class TestRemoveKeywords(unittest.TestCase):

    def test_remove_all_removes_all(self):
        suite = self._suite_with_setup_and_teardown_and_test_with_keywords()
        self._remove('ALL', suite)
        for keyword in chain((suite.setup, suite.teardown), suite.tests[0].body):
            self._should_contain_no_messages_or_keywords(keyword)

    def test_remove_passed_removes_from_passed_test(self):
        suite = TestSuite()
        test = suite.tests.create(status='PASS')
        test.body.create_keyword(status='PASS').body.create_message(message='keyword message')
        test.body.create_keyword(status='PASS').body.create_keyword(status='PASS')
        self._remove_passed(suite)
        for keyword in test.body:
            self._should_contain_no_messages_or_keywords(keyword)

    def test_remove_passed_removes_setup_and_teardown_from_passed_suite(self):
        suite = TestSuite()
        suite.tests.create(status='PASS')
        suite.setup.config(kwname='S', status='PASS').body.create_keyword()
        suite.teardown.config(kwname='T', status='PASS').body.create_message(message='message')
        self._remove_passed(suite)
        for keyword in suite.setup, suite.teardown:
            self._should_contain_no_messages_or_keywords(keyword)

    def test_remove_passed_does_not_remove_when_test_failed(self):
        suite = TestSuite()
        test = suite.tests.create(status='FAIL')
        test.body.create_keyword(status='PASS').body.create_keyword()
        test.body.create_keyword(status='PASS').body.create_message(message='message')
        failed_keyword = test.body.create_keyword(status='FAIL')
        failed_keyword.body.create_message('mess')
        failed_keyword.body.create_keyword()
        self._remove_passed(suite)
        assert_equal(len(test.body[0].body), 1)
        assert_equal(len(test.body[0].messages), 0)
        assert_equal(len(test.body[1].body), 1)
        assert_equal(len(test.body[1].messages), 1)
        assert_equal(len(test.body[2].body), 2)
        assert_equal(len(test.body[2].messages), 1)

    def test_remove_passed_does_not_remove_when_test_contains_warning(self):
        suite = TestSuite()
        test = self._test_with_warning(suite)
        self._remove_passed(suite)
        assert_equal(len(test.body[0].body), 1)
        assert_equal(len(test.body[1].messages), 1)

    def _test_with_warning(self, suite):
        test = suite.tests.create(status='PASS')
        test.body.create_keyword(status='PASS').body.create_keyword()
        test.body.create_keyword(status='PASS').body.create_message(message='danger!',
                                                                    level='WARN')
        return test

    def test_remove_passed_does_not_remove_setup_and_teardown_from_failed_suite(self):
        suite = TestSuite()
        suite.setup.config(kwname='SETUP').body.create_message(message='some')
        suite.teardown.config(type='TEARDOWN').body.create_keyword()
        suite.tests.create(status='FAIL')
        self._remove_passed(suite)
        assert_equal(len(suite.setup.messages), 1)
        assert_equal(len(suite.teardown.body), 1)

    def test_remove_for_removes_passed_iterations_except_last(self):
        suite, loop = self.suite_with_for_loop()
        last = loop.body[-1]
        self._remove_for_loop(suite)
        assert_equal(len(loop.body), 1)
        assert_true(loop.body[-1] is last)

    def suite_with_for_loop(self):
        suite = TestSuite()
        test = suite.tests.create(status='PASS')
        loop = test.body.create_for(status='PASS')
        for i in range(100):
            loop.body.create_iteration({'${i}': i}, status='PASS')\
                .body.create_keyword(kwname='k%d' % i, status='PASS')\
                .body.create_message(message='something')
        return suite, loop

    def test_remove_for_does_not_remove_failed_iterations(self):
        suite, loop = self.suite_with_for_loop()
        fail1 = loop.body[42]
        fail2 = loop.body[75]
        last = loop.body[-1]
        fail1.failed = fail2.failed = True
        self._remove_for_loop(suite)
        assert_equal(len(loop.body), 3)
        assert_equal(list(loop.body), [fail1, fail2, last])

    def test_remove_for_does_not_remove_iterations_with_warnings(self):
        suite, loop = self.suite_with_for_loop()
        loop.body[2].body.create_message(message='danger!', level='WARN')
        warn = loop.body[2]
        last = loop.body[-1]
        self._remove_for_loop(suite)
        assert_equal(len(loop.body), 2)
        assert_equal(list(loop.body), [warn, last])

    def test_remove_based_on_multiple_condition(self):
        suite = TestSuite()
        t1 = suite.tests.create(status='PASS')
        t1.body.create_keyword().body.create_message()
        t2 = suite.tests.create(status='FAIL')
        t2.body.create_keyword().body.create_message()
        iteration = t2.body.create_for().body.create_iteration()
        for i in range(10):
            iteration.body.create_keyword(status='PASS')
        self._remove(['passed', 'for'], suite)
        assert_equal(len(t1.body[0].messages), 0)
        assert_equal(len(t2.body[0].messages), 1)
        assert_equal(len(t2.body[1].body), 1)

    def _suite_with_setup_and_teardown_and_test_with_keywords(self):
        suite = TestSuite()
        suite.setup.config(kwname='S', status='PASS').body.create_message('setup message')
        suite.teardown.config(kwname='T', status='PASS').body.create_message(message='message')
        test = suite.tests.create()
        test.body.create_keyword().body.create_keyword()
        test.body.create_keyword().body.create_message('kw with message')
        return suite

    def _should_contain_no_messages_or_keywords(self, keyword):
        assert_equal(len(keyword.messages), 0)
        assert_equal(len(keyword.body), 0)

    def _remove(self, option, item):
        item.visit(SuiteConfigurer(remove_keywords=option))

    def _remove_passed(self, item):
        self._remove('PASSED', item)

    def _remove_for_loop(self, item):
        self._remove('FOR', item)


if __name__ == '__main__':
    unittest.main()



import unittest

from robot.htmldata.template import HtmlTemplate
from robot.htmldata import LOG, REPORT
from robot.utils.asserts import assert_true, assert_equal, assert_raises


class TestHtmlTemplate(unittest.TestCase):

    def test_creating(self):
        log = list(HtmlTemplate(LOG))
        assert_true(log[0].startswith('<!DOCTYPE'))
        assert_equal(log[-1], '</html>')

    def test_lines_do_not_have_line_breaks(self):
        for line in HtmlTemplate(REPORT):
            assert_true(not line.endswith('\n'))

    def test_bad_path(self):
        assert_raises(ValueError, HtmlTemplate, 'one_part.html')
        assert_raises(ValueError, HtmlTemplate, 'more_than/two/parts.html')

    def test_non_existing(self):
        assert_raises((ImportError, IOError), list, HtmlTemplate('non/ex.html'))


if __name__ == "__main__":
    unittest.main()



from robot.libraries.BuiltIn import BuiltIn


class BuiltInPropertys:

    def __init__(self, dry_run=False):
        assert BuiltIn().robot_running is True
        assert BuiltIn().dry_run_active is dry_run

    def keyword(self):
        assert BuiltIn().robot_running is True
        assert BuiltIn().dry_run_active is False



from datetime import timedelta, datetime
import time

TIMEZONE = time.altzone if time.localtime().tm_isdst else time.timezone
EPOCH = 1542892422.0 + time.timezone


def all_days_for_year(year):
    year = int(year)
    dt = datetime(year, 1, 1)
    day = timedelta(days=1)
    while dt.year == year:
        yield dt.strftime('%Y-%m-%d %H:%M:%S')
        dt += day


def year_range(start, end, step=1, format='timestamp'):
    dt = datetime(int(start), 1, 1)
    end = int(end)
    step = int(step)
    while dt.year <= end:
        if format == 'datetime':
            yield dt
        if format == 'timestamp':
            yield dt.strftime('%Y-%m-%d %H:%M:%S')
        if format == 'epocn':
            yield time.mktime(dt.timetuple())
        dt = dt.replace(year=dt.year + step)



from robot.api.deco import keyword


class Embedded:

    def __init__(self):
        self.called = 0

    @keyword('Called ${times} time(s)', types={'times': int})
    def called_times(self, times):
        self.called += 1
        if self.called != times:
            raise AssertionError('Called %s time(s), expected %s time(s).'
                                 % (self.called, times))



import unittest

from robot.utils.asserts import assert_equal, assert_none
from robot.model.tagstatistics import TagStatisticsBuilder, TagStatLink
from robot.model import Tags
from robot.result import TestCase
from robot.utils import MultiMatcher


class TestTagStatistics(unittest.TestCase):
    _incl_excl_data = [([], []),
                       ([], ['t1','t2']),
                       (['t1'], ['t1','t2']),
                       (['t1','t2'], ['t1','t2','t3','t4']),
                       (['UP'], ['t1','t2','up']),
                       (['not','not2'], ['t1','t2','t3']),
                       (['t*'], ['t1','s1','t2','t3','s2','s3']),
                       (['T*','r'], ['t1','t2','r','teeeeeeee']),
                       (['*'], ['t1','t2','s1','tag']),
                       (['t1','t2','t3','not'], ['t1','t2','t3','t4','s1','s2'])]

    def test_include(self):
        for incl, tags in self._incl_excl_data:
            builder = TagStatisticsBuilder(included=incl)
            builder.add_test(TestCase(status='PASS', tags=tags))
            matcher = MultiMatcher(incl, match_if_no_patterns=True)
            expected = [tag for tag in tags if matcher.match(tag)]
            assert_equal([s.name for s in builder.stats], sorted(expected))

    def test_exclude(self):
        for excl, tags in self._incl_excl_data:
            builder = TagStatisticsBuilder(excluded=excl)
            builder.add_test(TestCase(status='PASS', tags=tags))
            matcher = MultiMatcher(excl)
            expected = [tag for tag in tags if not matcher.match(tag)]
            assert_equal([s.name for s in builder.stats], sorted(expected))

    def test_include_and_exclude(self):
        for incl, excl, tags, exp in [
               ([], [], ['t0','t1','t2'], ['t0','t1','t2']),
               (['t1'], ['t2'], ['t0','t1','t2'], ['t1']),
               (['t?'], ['t2'], ['t0','t1','t2','x'], ['t0','t1']),
               (['t?'], ['*2'], ['t0','t1','t2','x2'], ['t0','t1']),
               (['t1','t2'], ['t2'], ['t0','t1','t2'], ['t1']),
               (['t1','t2','t3','not'], ['t2','t0'],
                ['t0','t1','t2','t3','x'], ['t1','t3'] )
              ]:
            builder = TagStatisticsBuilder(included=incl, excluded=excl)
            builder.add_test(TestCase(status='PASS', tags=tags))
            assert_equal([s.name for s in builder.stats], exp),

    def test_combine_with_name(self):
        for comb_tags, expected_name in [
                ([], ''),
                ([('t1&t2', 'my name')], 'my name'),
                ([('t1NOTt3', 'Others')], 'Others'),
                ([('1:2&2:3', 'nAme')], 'nAme'),
                ([('3*', '')], '3*'),
                ([('4NOT5', 'Some new name')], 'Some new name')
               ]:
            builder = TagStatisticsBuilder(combined=comb_tags)
            assert_equal(bool(list(builder.stats)), bool(expected_name))
            if expected_name:
                assert_equal([s.name for s in builder.stats], [expected_name])

    def test_is_combined_with_and_statements(self):
        for comb_tags, test_tags, expected_count in [
                ('t1', ['t1'], 1),
                ('t1', ['t2'], 0),
                ('t1&t2', ['t1'], 0),
                ('t1&t2', ['t1','t2'], 1),
                ('t1&t2', ['T1','t 2','t3'], 1),
                ('t*', ['s','t','u'], 1),
                ('t*', ['s','tee','t'], 1),
                ('t*&s', ['s','tee','t'], 1),
                ('t*&s&non', ['s','tee','t'], 0)
               ]:
            self._verify_combined_statistics(comb_tags, test_tags, expected_count)

    def _verify_combined_statistics(self, comb_tags, test_tags, expected_count):
        builder = TagStatisticsBuilder(combined=[(comb_tags, 'name')])
        builder.add_test(TestCase(tags=test_tags))
        assert_equal([s.total for s in builder.stats if s.combined], [expected_count])

    def test_is_combined_with_not_statements(self):
        for comb_tags, test_tags, expected_count in [
                ('t1NOTt2', [], 0),
                ('t1NOTt2', ['t1'], 1),
                ('t1NOTt2', ['t1','t2'], 0),
                ('t1NOTt2', ['t3'], 0),
                ('t1NOTt2', ['t3','t2'], 0),
                ('t*NOTt2', ['t1'], 1),
                ('t*NOTt2', ['t'], 1),
                ('t*NOTt2', ['TEE'], 1),
                ('t*NOTt2', ['T2'], 0),
                ('T*NOTT?', ['t'], 1),
                ('T*NOTT?', ['tt'], 0),
                ('T*NOTT?', ['ttt'], 1),
                ('T*NOTT?', ['tt','t'], 0),
                ('T*NOTT?', ['ttt','something'], 1),
                ('tNOTs*NOTr', ['t'], 1),
                ('tNOTs*NOTr', ['t','s'], 0),
                ('tNOTs*NOTr', ['S','T'], 0),
                ('tNOTs*NOTr', ['R','T','s'], 0),
                ('*NOTt', ['t'], 0),
                ('*NOTt', ['e'], 1),
                ('*NOTt', [], 0),
                ]:
            self._verify_combined_statistics(comb_tags, test_tags, expected_count)

    def test_starting_with_not(self):
        for comb_tags, test_tags, expected_count in [
                ('NOTt', ['t'], 0),
                ('NOTt', ['e'], 1),
                ('NOTt', [], 1),
                ('NOTtORe', ['e'], 0),
                ('NOTtORe', ['e', 't'], 0),
                ('NOTtORe', ['h'], 1),
                ('NOTtORe', [], 1),
                ('NOTtANDe', [], 1),
                ('NOTtANDe', ['t'], 1),
                ('NOTtANDe', ['t', 'e'], 0),
                ('NOTtNOTe', ['t', 'e'], 0),
                ('NOTtNOTe', ['t'], 0),
                ('NOTtNOTe', ['e'], 0),
                ('NOTtNOTe', ['d'], 1),
                ('NOTtNOTe', [], 1),
                ('NOT*', ['t'], 0),
                ('NOT*', [], 1),
                ]:
            self._verify_combined_statistics(comb_tags, test_tags, expected_count)

    def test_combine_with_same_name_as_existing_tag(self):
        builder = TagStatisticsBuilder(combined=[('x*', 'name')])
        builder.add_test(TestCase(tags=['name', 'another']))
        assert_equal([(s.name, s.combined) for s in builder.stats],
                      [('name', 'x*'),
                       ('another', None),
                       ('name', None)])

    def test_iter(self):
        builder = TagStatisticsBuilder()
        assert_equal(list(builder.stats), [])
        builder.add_test(TestCase())
        assert_equal(list(builder.stats), [])
        builder.add_test(TestCase(tags=['a']))
        assert_equal(len(list(builder.stats)), 1)
        builder.add_test(TestCase(tags=['A', 'B']))
        assert_equal(len(list(builder.stats)), 2)

    def test_iter_sorting(self):
        builder = TagStatisticsBuilder(combined=[('c*', ''), ('xxx', 'a title')])
        builder.add_test(TestCase(tags=['c1', 'c2', 't1']))
        builder.add_test(TestCase(tags=['c1', 'n2', 't2']))
        builder.add_test(TestCase(tags=['n1', 'n2', 't1', 't3']))
        assert_equal([(s.name, s.info, s.total) for s in builder.stats],
                     [('a title', 'combined', 0),
                      ('c*', 'combined', 2),
                      ('c1', '', 2),
                      ('c2', '', 1),
                      ('n1', '', 1),
                      ('n2', '', 2),
                      ('t1', '', 2),
                      ('t2', '', 1),
                      ('t3', '', 1)])

    def test_combine(self):
        # This is more like an acceptance test than a unit test ...
        for comb_tags, tests_tags in [
                (['t1&t2'], [['t1','t2','t3'],['t1','t3']]),
                (['1&2&3'], [['1','2','3'],['1','2','3','4']]),
                (['1&2','1&3'], [['1','2','3'],['1','3'],['1']]),
                (['t*'], [['t1','x','y'],['tee','z'],['t']]),
                (['t?&s'], [['t1','s'],['tt','s','u'],['tee','s']]),
                (['t*&s','*'], [['s','t','u'],['tee','s'],[],['x']]),
                (['tNOTs'], [['t','u'],['t','s']]),
                (['tNOTs','t&s','tNOTsNOTu', 't&sNOTu'],
                  [['t','u'],['t','s'],['s','t','u'],['t'],['t','v']]),
                (['nonex'], [['t1'],['t1,t2'],[]])
                ]:
            # 1) Create tag stats
            builder = TagStatisticsBuilder(combined=[(t, '') for t in comb_tags])
            all_tags = []
            for tags in tests_tags:
                builder.add_test(TestCase(status='PASS', tags=tags),)
                all_tags.extend(tags)
            # 2) Actual values
            names = [stat.name for stat in builder.stats]
            # 3) Expected values
            exp_names = sorted(comb_tags) + sorted(list(set(all_tags)))
            # 4) Verify names (match counts were already verified)
            assert_equal(names, exp_names)


class TestTagStatDoc(unittest.TestCase):

    def test_simple(self):
        builder = TagStatisticsBuilder(docs=[('t1', 'doc')])
        builder.add_test(TestCase(tags=['t1', 't2']))
        builder.add_test(TestCase(tags=['T 1']))
        builder.add_test(TestCase(tags=['T_1'], status='PASS'))
        self._verify_stats(builder.stats.tags['t1'], 'doc', 2, 1)

    def test_pattern(self):
        builder = TagStatisticsBuilder(docs=[('t?', '*doc*')])
        builder.add_test(TestCase(tags=['t1', 'T2']))
        builder.add_test(TestCase(tags=['_t__1_', 'T 3']))
        self._verify_stats(builder.stats.tags['t1'], '*doc*', 2)
        self._verify_stats(builder.stats.tags['t2'], '*doc*', 1)
        self._verify_stats(builder.stats.tags['t3'], '*doc*', 1)

    def test_multiple_matches(self):
        builder = TagStatisticsBuilder(docs=[('t_1', 'd1'), ('t?', 'd2')])
        builder.add_test(TestCase(tags=['t1', 't_2']))
        self._verify_stats(builder.stats.tags['t1'], 'd1 & d2', 1)
        self._verify_stats(builder.stats.tags['t2'], 'd2', 1)

    def _verify_stats(self, stat, doc, failed, passed=0, combined=None):
        assert_equal(stat.doc, doc)
        assert_equal(stat.failed, failed)
        assert_equal(stat.passed, passed)
        assert_equal(stat.total, passed + failed)
        assert_equal(stat.combined, combined)


class TestTagStatLink(unittest.TestCase):

    def test_valid_string_is_parsed_correctly(self):
        for arg, exp in [(('Tag', 'bar/foo.html', 'foobar'),
                          ('^Tag$', 'bar/foo.html', 'foobar')),
                         (('hello', 'gopher://hello.world:8090/hello.html',
                           'Hello World'),
                          ('^hello$', 'gopher://hello.world:8090/hello.html',
                           'Hello World'))]:
            link = TagStatLink(*arg)
            assert_equal(exp[0], link._regexp.pattern)
            assert_equal(exp[1], link._link)
            assert_equal(exp[2], link._title)

    def test_valid_string_containing_patterns_is_parsed_correctly(self):
        for arg, exp_pattern in [('*', '^(.*)$'), ('f*r', '^f(.*)r$'),
                                 ('*a*', '^(.*)a(.*)$'),  ('?', '^(.)$'),
                                 ('??', '^(..)$'), ('f???ar', '^f(...)ar$'),
                                 ('F*B?R*?', '^F(.*)B(.)R(.*)(.)$')]:
            link = TagStatLink(arg, 'some_url', 'some_title')
            assert_equal(exp_pattern, link._regexp.pattern)

    def test_underscores_in_title_are_converted_to_spaces(self):
        link = TagStatLink('', '', 'my_name')
        assert_equal(link._title, 'my name')

    def test_get_link_returns_correct_link_when_matches(self):
        for arg, exp in [(('smoke', 'http://tobacco.com', 'Lung_cancer'),
                          ('http://tobacco.com', 'Lung cancer')),
                         (('tag', 'ftp://foo:809/bar.zap', 'Foo_in a Bar'),
                          ('ftp://foo:809/bar.zap', 'Foo in a Bar'))]:
            link = TagStatLink(*arg)
            assert_equal(exp, link.get_link(arg[0]))

    def test_get_link_returns_none_when_no_match(self):
        link = TagStatLink('smoke', 'http://tobacco.com', 'Lung cancer')
        for tag in ['foo', 'b a r', 's moke']:
            assert_none(link.get_link(tag))

    def test_pattern_matches_case_insensitively(self):
        exp = 'http://tobacco.com', 'Lung cancer'
        link = TagStatLink('smoke', *exp)
        for tag in ['Smoke', 'SMOKE', 'smoke']:
            assert_equal(exp, link.get_link(tag))

    def test_pattern_matches_when_spaces(self):
        exp = 'http://tobacco.com', 'Lung cancer'
        link = TagStatLink('smoking kills', *exp)
        for tag in ['Smoking Kills', 'SMOKING KILLS']:
            assert_equal(exp, link.get_link(tag))

    def test_pattern_match(self):
        link = TagStatLink('f?o*r', 'http://foo/bar.html', 'FooBar')
        for tag in ['foobar', 'foor', 'f_ofoobarfoobar', 'fOoBAr']:
            assert_equal(link.get_link(tag), ('http://foo/bar.html', 'FooBar'))

    def test_pattern_substitution_with_one_match(self):
        link = TagStatLink('tag-*', 'http://tracker/?id=%1', 'Tracker')
        for id in ['1', '23', '456']:
            exp = ('http://tracker/?id=%s' % id, 'Tracker')
            assert_equal(exp, link.get_link('tag-%s' % id))

    def test_pattern_substitution_with_multiple_matches(self):
        link = TagStatLink('?-*', 'http://tracker/?id=%1-%2', 'Tracker')
        for id1, id2 in [('1', '2'), ('3', '45'), ('f', 'bar')]:
            exp = ('http://tracker/?id=%s-%s' % (id1, id2), 'Tracker')
            assert_equal(exp, link.get_link('%s-%s' % (id1, id2)))

    def test_pattern_substitution_with_multiple_substitutions(self):
        link = TagStatLink('??-?-*', '%3-%3-%1-%2-%3', 'Tracker')
        assert_equal(link.get_link('aa-b-XXX'), ('XXX-XXX-aa-b-XXX', 'Tracker'))

    def test_matches_are_ignored_in_pattern_substitution(self):
        link = TagStatLink('???-*-*-?', '%4-%2-%2-%4', 'Tracker')
        assert_equal(link.get_link('AAA-XXX-ABC-B'), ('B-XXX-XXX-B', 'Tracker'))


if __name__ == "__main__":
    unittest.main()



import base64
import unittest
import zlib
from pathlib import Path

from robot.utils.asserts import assert_equal, assert_true
from robot.result import Keyword, Message, TestCase, TestSuite
from robot.result.executionerrors import ExecutionErrors
from robot.model import Statistics, BodyItem
from robot.reporting.jsmodelbuilders import (
    ErrorsBuilder, JsBuildingContext, KeywordBuilder, MessageBuilder,
    StatisticsBuilder, SuiteBuilder, TestBuilder
)
from robot.reporting.stringcache import StringIndex


CURDIR = Path(__file__).resolve().parent


def decode_string(string):
    return zlib.decompress(base64.b64decode(string.encode('ASCII'))).decode('UTF-8')


def remap(model, strings):
    if isinstance(model, StringIndex):
        if strings[model].startswith('*'):
            # Strip the asterisk from a raw string.
            return strings[model][1:]
        return decode_string(strings[model])
    elif isinstance(model, (int, type(None))):
        return model
    elif isinstance(model, tuple):
        return tuple(remap(item, strings) for item in model)
    else:
        raise AssertionError("Item '%s' has invalid type '%s'" % (model, type(model)))


class TestBuildTestSuite(unittest.TestCase):

    def test_default_suite(self):
        self._verify_suite(TestSuite())

    def test_suite_with_values(self):
        suite = TestSuite('Name', 'Doc', {'m1': 'v1', 'M2': 'V2'}, None, False, 'Message',
                          '20111204 19:00:00.000', '20111204 19:00:42.001')
        self._verify_suite(suite, 'Name', 'Doc', ('m1', '<p>v1</p>', 'M2', '<p>V2</p>'),
                           message='Message', start=0, elapsed=42001)

    def test_relative_source(self):
        self._verify_suite(TestSuite(source='non-existing'),
                           name='Non-Existing', source='non-existing')
        source = CURDIR / 'test_jsmodelbuilders.py'
        self._verify_suite(TestSuite(name='x', source=source),
                           name='x', source=str(source), relsource=str(source.name))

    def test_suite_html_formatting(self):
        self._verify_suite(TestSuite(name='*xxx*', doc='*bold* <&>',
                                     metadata={'*x*': '*b*', '<': '>'}),
                           name='*xxx*', doc='<b>bold</b> &lt;&amp;&gt;',
                           metadata=('*x*', '<p><b>b</b></p>', '&lt;', '<p>&gt;</p>'))

    def test_default_test(self):
        self._verify_test(TestCase())

    def test_test_with_values(self):
        test = TestCase('Name', '*Doc*', ['t1', 't2'], '1 minute', 42, 'PASS', 'Msg',
                        '20111204 19:22:22.222', '20111204 19:22:22.333')
        test.setup.config(kwname='setup')
        test.teardown.config(kwname='td')
        k1 = self._verify_keyword(test.setup, type=1, kwname='setup')
        k2 = self._verify_keyword(test.teardown, type=2, kwname='td')
        self._verify_test(test, 'Name', '<b>Doc</b>', ('t1', 't2'),
                          '1 minute', 1, 'Msg', 0, 111, (k1, k2))

    def test_name_escaping(self):
        kw = Keyword('quote:"', 'and *url* https://url.com', '*"Doc"*',)
        self._verify_keyword(kw, 0, 'quote:&quot;', 'and *url* https://url.com', '<b>"Doc"</b>')
        test = TestCase('quote:" and *url* https://url.com', '*"Doc"*',)
        self._verify_test(test, 'quote:&quot; and *url* https://url.com', '<b>"Doc"</b>')
        suite = TestSuite('quote:" and *url* https://url.com', '*"Doc"*',)
        self._verify_suite(suite, 'quote:&quot; and *url* https://url.com', '<b>"Doc"</b>')

    def test_default_keyword(self):
        self._verify_keyword(Keyword())

    def test_keyword_with_values(self):
        kw = Keyword('KW Name', 'libname', 'http://doc', ('arg1', 'arg2'),
                     ('${v1}', '${v2}'), ('tag1', 'tag2'), '1 second', 'SETUP',
                     'PASS', '20111204 19:42:42.000', '20111204 19:42:42.042')
        self._verify_keyword(kw, 1, 'KW Name', 'libname',
                             '<a href="http://doc">http://doc</a>',
                             'arg1, arg2', '${v1}, ${v2}', 'tag1, tag2',
                             '1 second', 1, 0, 42)

    def test_default_message(self):
        self._verify_message(Message())
        self._verify_min_message_level('INFO')

    def test_message_with_values(self):
        msg = Message('Message', 'DEBUG', timestamp='20111204 22:04:03.210')
        self._verify_message(msg, 'Message', 1, 0)
        self._verify_min_message_level('DEBUG')

    def test_warning_linking(self):
        msg = Message('Message', 'WARN', timestamp='20111204 22:04:03.210',
                      parent=TestCase().body.create_keyword())
        self._verify_message(msg, 'Message', 3, 0)
        links = self.context._msg_links
        assert_equal(len(links), 1)
        key = (msg.message, msg.level, msg.timestamp)
        assert_equal(remap(links[key], self.context.strings), 't1-k1')

    def test_error_linking(self):
        msg = Message('ERROR Message', 'ERROR', timestamp='20150609 01:02:03.004',
                      parent=TestCase().body.create_keyword().body.create_keyword())
        self._verify_message(msg, 'ERROR Message', 4, 0)
        links = self.context._msg_links
        assert_equal(len(links), 1)
        key = (msg.message, msg.level, msg.timestamp)
        assert_equal(remap(links[key], self.context.strings), 't1-k1-k1')

    def test_message_with_html(self):
        self._verify_message(Message('<img>'), '&lt;img&gt;')
        self._verify_message(Message('<b></b>', html=True), '<b></b>')

    def test_nested_structure(self):
        suite = TestSuite()
        suite.setup.config(kwname='setup')
        suite.teardown.config(kwname='td')
        K1 = self._verify_keyword(suite.setup, type=1, kwname='setup')
        K2 = self._verify_keyword(suite.teardown, type=2, kwname='td')
        suite.suites = [TestSuite()]
        suite.suites[0].tests = [TestCase(tags=['crit', 'xxx'])]
        t = self._verify_test(suite.suites[0].tests[0], tags=('crit', 'xxx'))
        suite.tests = [TestCase(), TestCase(status='PASS')]
        S1 = self._verify_suite(suite.suites[0],
                                status=0, tests=(t,), stats=(1, 0, 1, 0))
        suite.tests[0].body = [Keyword(type=Keyword.FOR), Keyword()]
        suite.tests[0].body[0].body = [Keyword(type=Keyword.ITERATION), Message()]
        k = self._verify_keyword(suite.tests[0].body[0].body[0], type=4)
        m = self._verify_message(suite.tests[0].body[0].messages[0])
        k1 = self._verify_keyword(suite.tests[0].body[0], type=3, body=(k, m))
        suite.tests[0].body[1].body = [Message(), Message('msg', level='TRACE')]
        m1 = self._verify_message(suite.tests[0].body[1].messages[0])
        m2 = self._verify_message(suite.tests[0].body[1].messages[1], 'msg', level=0)
        k2 = self._verify_keyword(suite.tests[0].body[1], body=(m1, m2))
        T1 = self._verify_test(suite.tests[0], body=(k1, k2))
        T2 = self._verify_test(suite.tests[1], status=1)
        self._verify_suite(suite, status=0, keywords=(K1, K2), suites=(S1,),
                           tests=(T1, T2), stats=(3, 1, 2, 0))
        self._verify_min_message_level('TRACE')

    def test_timestamps(self):
        suite = TestSuite(starttime='20111205 00:33:33.333')
        suite.setup.config(kwname='s1', starttime='20111205 00:33:33.334')
        suite.setup.body.create_message('Message', timestamp='20111205 00:33:33.343')
        suite.setup.body.create_message(level='DEBUG', timestamp='20111205 00:33:33.344')
        suite.tests.create(starttime='20111205 00:33:34.333')
        context = JsBuildingContext()
        model = SuiteBuilder(context).build(suite)
        self._verify_status(model[5], start=0)
        self._verify_status(model[-2][0][8], start=1)
        self._verify_mapped(model[-2][0][-1], context.strings,
                            ((10, 2, 'Message'), (11, 1, '')))
        self._verify_status(model[-3][0][4], start=1000)

    def test_if(self):
        test = TestSuite().tests.create()
        test.body.create_if()
        test.body[0].body.create_branch(BodyItem.IF, '$x > 0', status='NOT RUN')
        test.body[0].body.create_branch(BodyItem.ELSE_IF, '$x < 0', status='PASS')
        test.body[0].body.create_branch(BodyItem.ELSE, status='NOT RUN')
        test.body[0].body[-1].body.create_keyword('z')
        exp_if = (
            5, '$x &gt; 0', '', '', '', '', '', '', (3, None, 0), ()
        )
        exp_else_if = (
            6, '$x &lt; 0', '', '', '', '', '', '', (1, None, 0), ()
        )
        exp_else = (
            7, '', '', '', '', '', '', '', (3, None, 0),
            ((0, 'z', '', '', '', '', '', '', (0, None, 0), ()),)
        )
        self._verify_test(test, body=(exp_if, exp_else_if, exp_else))

    def test_for(self):
        test = TestSuite().tests.create()
        test.body.create_for(variables=['${x}'], values=['a', 'b'])
        test.body.create_for(['${x}'], 'IN ENUMERATE', ['a', 'b'], start='1')
        end = ('', '', '', '', '', '', (0, None, 0), ())
        exp_f1 = (3, '${x} IN [ a | b ]', *end)
        exp_f2 = (3, '${x} IN ENUMERATE [ a | b | start=1 ]', *end)
        self._verify_test(test, body=(exp_f1, exp_f2))

    def test_message_directly_under_test(self):
        test = TestSuite().tests.create()
        test.body.create_message('Hi from test')
        test.body.create_keyword().body.create_message('Hi from keyword')
        test.body.create_message('Hi from test again', 'WARN')
        exp_m1 = (None, 2, 'Hi from test')
        exp_kw = (0, '', '', '', '', '', '', '', (0, None, 0),
                  ((None, 2, 'Hi from keyword'),))
        exp_m3 = (None, 3, 'Hi from test again')
        self._verify_test(test, body=(exp_m1, exp_kw, exp_m3))

    def _verify_status(self, model, status=0, start=None, elapsed=0):
        assert_equal(model, (status, start, elapsed))

    def _verify_suite(self, suite, name='', doc='', metadata=(), source='',
                      relsource='', status=2, message='', start=None, elapsed=0,
                      suites=(), tests=(), keywords=(), stats=(0, 0, 0, 0)):
        status = (status, start, elapsed, message) \
                if message else (status, start, elapsed)
        doc = '<p>%s</p>' % doc if doc else ''
        return self._build_and_verify(SuiteBuilder, suite, name, source,
                                      relsource, doc, metadata, status,
                                      suites, tests, keywords, stats)

    def _get_status(self, *elements):
        return elements if elements[-1] else elements[:-1]

    def _verify_test(self, test, name='', doc='', tags=(), timeout='',
                     status=0, message='', start=None, elapsed=0, body=()):
        status = (status, start, elapsed, message) \
                if message else (status, start, elapsed)
        doc = '<p>%s</p>' % doc if doc else ''
        return self._build_and_verify(TestBuilder, test, name, timeout,
                                      doc, tags, status, body)

    def _verify_keyword(self, keyword, type=0, kwname='', libname='', doc='',
                        args='', assign='', tags='', timeout='', status=0,
                        start=None, elapsed=0, body=()):
        status = (status, start, elapsed)
        doc = '<p>%s</p>' % doc if doc else ''
        return self._build_and_verify(KeywordBuilder, keyword, type, kwname,
                                      libname, timeout, doc, args, assign, tags,
                                      status, body)

    def _verify_message(self, msg, message='', level=2, timestamp=None):
        return self._build_and_verify(MessageBuilder, msg, timestamp, level, message)

    def _verify_min_message_level(self, expected):
        assert_equal(self.context.min_level, expected)

    def _build_and_verify(self, builder_class, item, *expected):
        self.context = JsBuildingContext(log_path=CURDIR / 'log.html')
        model = builder_class(self.context).build(item)
        self._verify_mapped(model, self.context.strings, expected)
        return expected

    def _verify_mapped(self, model, strings, expected):
        mapped_model = tuple(remap(model, strings))
        assert_equal(mapped_model, expected)


class TestSplitting(unittest.TestCase):

    def test_test_keywords(self):
        suite = self._get_suite_with_tests()
        expected, _ = self._build_and_remap(suite)
        expected_split = [expected[-3][0][-1], expected[-3][1][-1]]
        expected[-3][0][-1], expected[-3][1][-1] = 1, 2
        model, context = self._build_and_remap(suite, split_log=True)
        assert_equal(context.strings, ('*', '*suite', '*t1', '*t2'))
        assert_equal(model, expected)
        assert_equal([strings for _, strings in context.split_results],
                      [('*', '*t1-k1', '*t1-k1-k1', '*t1-k2'), ('*', '*t2-k1')])
        assert_equal([self._to_list(remap(*res)) for res in context.split_results],
                      expected_split)

    def _get_suite_with_tests(self):
        suite = TestSuite(name='suite')
        suite.tests = [TestCase('t1'), TestCase('t2')]
        suite.tests[0].body = [Keyword('t1-k1'), Keyword('t1-k2')]
        suite.tests[0].body[0].body = [Keyword('t1-k1-k1')]
        suite.tests[1].body = [Keyword('t2-k1')]
        return suite

    def _build_and_remap(self, suite, split_log=False):
        context = JsBuildingContext(split_log=split_log)
        model = remap(SuiteBuilder(context).build(suite), context.strings)
        return self._to_list(model), context

    def _to_list(self, model):
        return list(self._to_list(item) if isinstance(item, tuple) else item
                    for item in model)

    def test_suite_keywords(self):
        suite = self._get_suite_with_keywords()
        expected, _ = self._build_and_remap(suite)
        expected_split = [expected[-2][0][-1], expected[-2][1][-1]]
        expected[-2][0][-1], expected[-2][1][-1] = 1, 2
        model, context = self._build_and_remap(suite, split_log=True)
        assert_equal(context.strings, ('*', '*root', '*k1', '*k2'))
        assert_equal(model, expected)
        assert_equal([strings for _, strings in context.split_results],
                     [('*', '*k1-k2'), ('*',)])
        assert_equal([self._to_list(remap(*res)) for res in context.split_results],
                     expected_split)

    def _get_suite_with_keywords(self):
        suite = TestSuite(name='root')
        suite.setup.config(kwname='k1')
        suite.teardown.config(kwname='k2')
        suite.setup.body.create_keyword('k1-k2')
        return suite

    def test_nested_suite_and_test_keywords(self):
        suite = self._get_nested_suite_with_tests_and_keywords()
        expected, _ = self._build_and_remap(suite)
        expected_split = [expected[-4][0][-3][0][-1], expected[-4][0][-3][1][-1],
                          expected[-4][1][-3][0][-1], expected[-4][1][-2][0][-1],
                          expected[-2][0][-1], expected[-2][1][-1]]
        (expected[-4][0][-3][0][-1], expected[-4][0][-3][1][-1],
         expected[-4][1][-3][0][-1], expected[-4][1][-2][0][-1],
         expected[-2][0][-1], expected[-2][1][-1]) = 1, 2, 3, 4, 5, 6
        model, context = self._build_and_remap(suite, split_log=True)
        assert_equal(model, expected)
        assert_equal([self._to_list(remap(*res)) for res in context.split_results],
                      expected_split)

    def _get_nested_suite_with_tests_and_keywords(self):
        suite = self._get_suite_with_keywords()
        sub = TestSuite(name='suite2')
        suite.suites = [self._get_suite_with_tests(), sub]
        sub.setup.config(kwname='kw')
        sub.setup.body.create_keyword('skw').body.create_message('Message')
        sub.tests.create('test', doc='tdoc').body.create_keyword('koowee', doc='kdoc')
        return suite

    def test_message_linking(self):
        suite = self._get_suite_with_keywords()
        msg1 = suite.setup.body[0].body.create_message(
            'Message 1', 'WARN', timestamp='20111204 22:04:03.210'
        )
        msg2 = suite.tests.create().body.create_keyword().body.create_message(
            'Message 2', 'ERROR', timestamp='20111204 22:04:04.210'
        )
        context = JsBuildingContext(split_log=True)
        SuiteBuilder(context).build(suite)
        errors = ErrorsBuilder(context).build(ExecutionErrors([msg1, msg2]))
        assert_equal(remap(errors, context.strings),
                     ((-1000, 3, 'Message 1', 's1-k1-k1'),
                      (0, 4, 'Message 2', 's1-t1-k1')))
        assert_equal(remap(context.link(msg1), context.strings), 's1-k1-k1')
        assert_equal(remap(context.link(msg2), context.strings), 's1-t1-k1')
        assert_true('*s1-k1-k1' in context.strings)
        assert_true('*s1-t1-k1' in context.strings)
        for res in context.split_results:
            assert_true('*s1-k1-k1' not in res[1])
            assert_true('*s1-t1-k1' not in res[1])


class TestPruneInput(unittest.TestCase):

    def setUp(self):
        self.suite = TestSuite()
        self.suite.setup.config(kwname='s')
        self.suite.teardown.config(kwname='t')
        s1 = self.suite.suites.create()
        s1.setup.config(kwname='s1')
        tc = s1.tests.create()
        tc.setup.config(kwname='tcs')
        tc.teardown.config(kwname='tct')
        tc.body = [Keyword(), Keyword(), Keyword()]
        tc.body[0].body = [Keyword(), Keyword(), Message(), Message(), Message()]
        tc.body[0].teardown.config(kwname='kt')
        s2 = self.suite.suites.create()
        t1 = s2.tests.create()
        t2 = s2.tests.create()
        t1.body = [Keyword()]
        t2.body = [Keyword(), Keyword()]

    def test_no_pruning(self):
        SuiteBuilder(JsBuildingContext(prune_input=False)).build(self.suite)
        assert_equal(self.suite.setup.kwname, 's')
        assert_equal(self.suite.teardown.kwname, 't')
        assert_equal(self.suite.suites[0].setup.kwname, 's1')
        assert_equal(self.suite.suites[0].teardown.kwname, None)
        assert_equal(self.suite.suites[0].tests[0].setup.kwname, 'tcs')
        assert_equal(self.suite.suites[0].tests[0].teardown.kwname, 'tct')
        assert_equal(len(self.suite.suites[0].tests[0].body), 3)
        assert_equal(len(self.suite.suites[0].tests[0].body[0].body), 5)
        assert_equal(len(self.suite.suites[0].tests[0].body[0].messages), 3)
        assert_equal(self.suite.suites[0].tests[0].body[0].teardown.kwname, 'kt')
        assert_equal(len(self.suite.suites[1].tests[0].body), 1)
        assert_equal(len(self.suite.suites[1].tests[1].body), 2)

    def test_prune_suites_from_suite(self):
        suite = self.suite
        assert_equal(len(suite.suites), 2)
        assert_equal(len(suite.tests), 0)
        SuiteBuilder(JsBuildingContext(prune_input=True)).build(suite)
        assert_equal(len(suite.suites), 0)
        assert_equal(len(suite.tests), 0)

    def test_prune_test_from_suite(self):
        suite = self.suite.suites[0]
        assert_equal(len(suite.suites), 0)
        assert_equal(len(suite.tests), 1)
        SuiteBuilder(JsBuildingContext(prune_input=True)).build(suite)
        assert_equal(len(suite.suites), 0)
        assert_equal(len(suite.tests), 0)

    def test_prune_test(self):
        test = self.suite.suites[0].tests[0]
        assert_equal(len(test.body), 3)
        TestBuilder(JsBuildingContext(prune_input=True)).build(test)
        assert_equal(len(test.body), 0)

    def test_prune_keyword(self):
        kw = self.suite.suites[0].tests[0].body[0]
        assert_equal(len(kw.body), 5)
        assert_equal(len(kw.messages), 3)
        KeywordBuilder(JsBuildingContext(prune_input=True)).build(kw)
        assert_equal(len(kw.body), 0)
        assert_equal(len(kw.messages), 0)

    def test_prune_errors(self):
        errors = ExecutionErrors([Message(), Message()])
        ErrorsBuilder(JsBuildingContext(prune_input=False)).build(errors)
        assert_equal(len(errors), 2)
        ErrorsBuilder(JsBuildingContext(prune_input=True)).build(errors)
        assert_equal(len(errors), 0)


class TestBuildStatistics(unittest.TestCase):

    def test_total_stats(self):
        all = self._build_statistics()[0][0]
        self._verify_stat(all, 2, 2, 1, 'All Tests', '00:00:33')

    def test_tag_stats(self):
        stats = self._build_statistics()[1]
        comb, t1, t2, t3 = self._build_statistics()[1]
        self._verify_stat(t2, 2, 0, 0, 't2', '00:00:22',
                          doc='doc', links='t:url')
        self._verify_stat(comb, 2, 0, 0, 'name', '00:00:22',
                          info='combined', combined='t1&amp;t2')
        self._verify_stat(t1, 2, 2, 0, 't1', '00:00:33')
        self._verify_stat(t3, 0, 1, 1, 't3', '00:00:01')

    def test_suite_stats(self):
        root, sub1, sub2 = self._build_statistics()[2]
        self._verify_stat(root, 2, 2, 1, 'root', '00:00:42', name='root', id='s1')
        self._verify_stat(sub1, 1, 1, 1, 'root.sub1', '00:00:10', name='sub1', id='s1-s1')
        self._verify_stat(sub2, 1, 1, 0, 'root.sub2', '00:00:30', name='sub2', id='s1-s2')

    def _build_statistics(self):
        return StatisticsBuilder().build(self._get_statistics())

    def _get_statistics(self):
        return Statistics(self._get_suite(),
                          suite_stat_level=2,
                          tag_stat_combine=[('t1&t2', 'name')],
                          tag_doc=[('t2', 'doc')],
                          tag_stat_link=[('?2', 'url', '%1')])

    def _get_suite(self):
        ts = lambda s, ms=0: '20120816 16:09:%02d.%03d' % (s, ms)
        suite = TestSuite(name='root', starttime=ts(0), endtime=ts(42))
        sub1 = TestSuite(name='sub1', starttime=ts(0), endtime=ts(10))
        sub2 = TestSuite(name='sub2')
        suite.suites = [sub1, sub2]
        sub1.tests = [
            TestCase(tags=['t1', 't2'], status='PASS', starttime=ts(0), endtime=ts(1, 500)),
            TestCase(tags=['t1', 't3'], status='FAIL', starttime=ts(2), endtime=ts(3, 499)),
            TestCase(tags=['t3'], status='SKIP', starttime=ts(3, 560), endtime=ts(3, 560))
        ]
        sub2.tests = [
            TestCase(tags=['t1', 't2'], status='PASS', starttime=ts(10), endtime=ts(30))
        ]
        sub2.suites.create(name='below suite stat level')\
                .tests.create(tags=['t1'], status='FAIL', starttime=ts(30), endtime=ts(40))
        return suite

    def _verify_stat(self, stat, pass_, fail, skip, label, elapsed, **attrs):
        attrs.update({'pass': pass_, 'fail': fail, 'skip': skip,
                      'label': label, 'elapsed': elapsed})
        assert_equal(stat, attrs)


class TestBuildErrors(unittest.TestCase):

    def setUp(self):
        msgs = [Message('Error', 'ERROR', timestamp='20111206 14:33:00.000'),
                Message('Warning', 'WARN', timestamp='20111206 14:33:00.042')]
        self.errors = ExecutionErrors(msgs)

    def test_errors(self):
        context = JsBuildingContext()
        model = ErrorsBuilder(context).build(self.errors)
        model = remap(model, context.strings)
        assert_equal(model, ((0, 4, 'Error'), (42, 3, 'Warning')))

    def test_linking(self):
        self.errors.messages.create('Linkable', 'WARN',
                                    timestamp='20111206 14:33:00.001')
        context = JsBuildingContext()
        msg = TestSuite().tests.create().body.create_keyword().body.create_message(
            'Linkable', 'WARN', timestamp='20111206 14:33:00.001'
        )
        MessageBuilder(context).build(msg)
        model = ErrorsBuilder(context).build(self.errors)
        model = remap(model, context.strings)
        assert_equal(model, ((-1, 4, 'Error'),
                             (41, 3, 'Warning'),
                             (0, 3, 'Linkable', 's1-t1-k1')))


if __name__ == '__main__':
    unittest.main()



import os
import time


class ListenAll:
    ROBOT_LISTENER_API_VERSION = '2'

    def __init__(self, *path):
        path = ':'.join(path) if path else self._get_default_path()
        self.outfile = open(path, 'w')
        self.start_attrs = []

    def _get_default_path(self):
        return os.path.join(os.getenv('TEMPDIR'), 'listen_all.txt')

    def start_suite(self, name, attrs):
        metastr = ' '.join('%s: %s' % (k, v) for k, v in attrs['metadata'].items())
        self.outfile.write("SUITE START: %s (%s) '%s' [%s]\n"
                           % (name, attrs['id'], attrs['doc'], metastr))
        self.start_attrs.append(attrs)

    def start_test(self, name, attrs):
        tags = [str(tag) for tag in attrs['tags']]
        self.outfile.write("TEST START: %s (%s, line %d) '%s' %s\n"
                           % (name, attrs['id'], attrs['lineno'], attrs['doc'], tags))
        self.start_attrs.append(attrs)

    def start_keyword(self, name, attrs):
        if attrs['assign']:
            assign = '%s = ' % ', '.join(attrs['assign'])
        else:
            assign = ''
        name = name + ' ' if name else ''
        if attrs['args']:
            args = '%s ' % [str(a) for a in attrs['args']]
        else:
            args = ''
        self.outfile.write("%s START: %s%s%s(line %d)\n"
                           % (attrs['type'], assign, name, args, attrs['lineno']))
        self.start_attrs.append(attrs)

    def log_message(self, message):
        msg, level = self._check_message_validity(message)
        if level != 'TRACE' and 'Traceback' not in msg:
            self.outfile.write('LOG MESSAGE: [%s] %s\n' % (level, msg))

    def message(self, message):
        msg, level = self._check_message_validity(message)
        if 'Settings' in msg:
            self.outfile.write('Got settings on level: %s\n' % level)

    def _check_message_validity(self, message):
        if message['html'] not in ['yes', 'no']:
            self.outfile.write('Log message has invalid `html` attribute %s' %
                               message['html'])
        if not message['timestamp'].startswith(str(time.localtime()[0])):
            self.outfile.write('Log message has invalid timestamp %s' %
                               message['timestamp'])
        return message['message'], message['level']

    def end_keyword(self, name, attrs):
        kw_type = 'KW' if attrs['type'] == 'Keyword' else attrs['type'].upper()
        self.outfile.write("%s END: %s\n" % (kw_type, attrs['status']))
        self._validate_start_attrs_at_end(attrs)

    def _validate_start_attrs_at_end(self, end_attrs):
        start_attrs = self.start_attrs.pop()
        for key in start_attrs:
            assert end_attrs[key] == start_attrs[key]

    def end_test(self, name, attrs):
        if attrs['status'] == 'PASS':
            self.outfile.write('TEST END: PASS\n')
        else:
            self.outfile.write("TEST END: %s %s\n"
                               % (attrs['status'], attrs['message']))
        self._validate_start_attrs_at_end(attrs)

    def end_suite(self, name, attrs):
        self.outfile.write('SUITE END: %s %s\n'
                           % (attrs['status'], attrs['statistics']))
        self._validate_start_attrs_at_end(attrs)

    def output_file(self, path):
        self._out_file('Output', path)

    def report_file(self, path):
        self._out_file('Report', path)

    def log_file(self, path):
        self._out_file('Log', path)

    def debug_file(self, path):
        self._out_file('Debug', path)

    def _out_file(self, name, path):
        assert os.path.isabs(path)
        self.outfile.write('%s: %s\n' % (name, os.path.basename(path)))

    def close(self):
        self.outfile.write('Closing...\n')
        self.outfile.close()



import unittest
import sys
import logging

from robot.utils.asserts import assert_equal, assert_true
from robot.api import logger


class MyStream:

    def __init__(self):
        self.flushed = False
        self.text = ''

    def write(self, text):
        self.text += text

    def flush(self):
        self.flushed = True


class TestConsole(unittest.TestCase):

    def setUp(self):
        self.original_stdout = sys.__stdout__
        sys.__stdout__ = self.stdout = MyStream()
        self.original_stderr = sys.__stderr__
        sys.__stderr__ = self.stderr = MyStream()

    def tearDown(self):
        sys.__stdout__ = self.original_stdout
        sys.__stderr__ = self.original_stderr

    def test_automatic_newline(self):
        logger.console('foo')
        self._verify('foo\n')

    def test_flushing(self):
        logger.console('foo', newline=False)
        self._verify('foo')
        assert_true(self.stdout.flushed)

    def test_streams(self):
        logger.console('to stdout', stream='stdout')
        logger.console('to stderr', stream='stdERR')
        logger.console('to stdout too', stream='invalid')
        self._verify('to stdout\nto stdout too\n', 'to stderr\n')

    def _verify(self, stdout='', stderr=''):
        assert_equal(self.stdout.text, stdout)
        assert_equal(self.stderr.text, stderr)


class MockHandler(logging.Handler):

    def __init__(self):
        logging.Handler.__init__(self)
        self.messages = []

    def emit(self, record):
        self.messages.append(record.getMessage())


class TestRedirectToPythonLogging(unittest.TestCase):

    def setUp(self):
        self.handler = MockHandler()
        root = logging.getLogger()
        root.addHandler(self.handler)
        root.setLevel(logging.NOTSET)

    def tearDown(self):
        logging.getLogger().removeHandler(self.handler)

    def test_logged_to_python(self):
        logger.info("Foo")
        logger.debug("Boo")
        logger.trace("Goo")
        logger.write("Doo", 'INFO')
        assert_equal(self.handler.messages, ['Foo', 'Boo', 'Goo', 'Doo'])

    def test_logger_to_python_with_html(self):
        logger.info("Foo", html=True)
        logger.write("Doo", 'INFO', html=True)
        logger.write("Joo", 'HTML')
        assert_equal(self.handler.messages, ['Foo', 'Doo', 'Joo'])

    def test_logger_to_python_with_console(self):
        logger.write("Foo", 'CONSOLE')
        assert_equal(self.handler.messages, ['Foo'])

if __name__ == '__main__':
    unittest.main()





class RecLibrary2:

    def keyword_only_in_library_2(self):
        print("Keyword from library 2")

    def keyword_in_both_libraries(self):
        print("Keyword from library 2")

    def keyword_in_all_resources_and_libraries(self):
        print("Keyword from library 2")

    def keyword_everywhere(self):
        print("Keyword from library 2")

    def no_operation(self):
        print("Overrides keyword from BuiltIn library")

    def similar_kw_4(self):
        pass



import unittest

from robot.utils.asserts import assert_equal, assert_false, assert_true
from robot.utils import (PY2, PY3, StringIO, JYTHON, IRONPYTHON, py2to3, py3to2,
                         is_unicode, platform, roundup, unicode, unic)


class TestCompatibilityLayer(unittest.TestCase):

    def test_constants(self):
        assert_true(PY3 is True)
        for not_supported in PY2, JYTHON, IRONPYTHON:
            assert_true(not_supported is False)

    def test_py2_under_platform(self):
        # https://github.com/robotframework/SSHLibrary/issues/401
        assert_true(platform.PY2 is False)

    def test_py2to3(self):
        @py2to3
        class X:
            def __unicode__(self):
                return 'Hyvä!'
            def __nonzero__(self):
                return False

        assert_false(X())
        assert_equal(str(X()), 'Hyvä!')

    def test_py3to2(self):
        @py3to2
        class X:
            def __str__(self):
                return 'Hyvä!'
            def __bool__(self):
                return False

        assert_false(X())
        assert_equal(str(X()), 'Hyvä!')

    def test_is_unicode(self):
        assert_true(is_unicode('Hyvä'))
        assert_true(is_unicode('Paha'))
        assert_false(is_unicode(b'xxx'))
        assert_false(is_unicode(42))

    def test_roundup(self):
        assert_true(roundup is round)

    def test_unicode(self):
        assert_true(unicode is str)

    def test_unic(self):
        assert_equal(unic('Hyvä'), 'Hyvä')
        assert_equal(unic('Paha'), 'Paha')
        assert_equal(unic(42), '42')
        assert_equal(unic(b'Hyv\xe4'), r'Hyv\xe4')
        assert_equal(unic(b'Paha'), 'Paha')

    def test_stringio(self):
        import io
        assert_true(StringIO is io.StringIO)


if __name__ == '__main__':
    unittest.main()



import os

try:
    basestring
except NameError:
    basestring = str


class ListenImports:
    ROBOT_LISTENER_API_VERSION = 2

    def __init__(self, imports):
        self.imports = open(imports, 'w')

    def library_import(self, name, attrs):
        self._imported("Library", name, attrs)

    def resource_import(self, name, attrs):
        self._imported("Resource", name, attrs)

    def variables_import(self, name, attrs):
        self._imported("Variables", name, attrs)

    def _imported(self, import_type, name, attrs):
        self.imports.write("Imported %s\n\tname: %s\n" % (import_type, name))
        for name in sorted(attrs):
            self.imports.write("\t%s: %s\n" % (name, self._pretty(attrs[name])))

    def _pretty(self, entry):
        if isinstance(entry, list):
            return '[%s]' % ', '.join(entry)
        if isinstance(entry, basestring) and os.path.isabs(entry):
            entry = entry.replace('$py.class', '.py').replace('.pyc', '.py')
            tokens = entry.split(os.sep)
            index = -1 if tokens[-1] != '__init__.py' else -2
            return '//' + '/'.join(tokens[index:])
        return entry

    def close(self):
        self.imports.close()



import unittest
from pathlib import Path

from robot.errors import DataError
from robot.utils import Importer
from robot.utils.asserts import assert_equal, assert_raises, assert_true
from robot.running import TestSuite, TestSuiteBuilder


DATADIR = (Path(__file__).parent / '../../atest/testdata/misc').resolve()


def build(*paths, **config):
    paths = [Path(DATADIR, p).resolve() for p in paths]
    suite = TestSuiteBuilder(**config).build(*paths)
    assert_true(isinstance(suite, TestSuite))
    assert_equal(suite.source, paths[0] if len(paths) == 1 else None)
    return suite


def assert_keyword(kw, assign=(), name='', args=(), type='KEYWORD'):
    assert_equal(kw.name, name)
    assert_equal(kw.args, args)
    assert_equal(kw.assign, assign)
    assert_equal(kw.type, type)


class TestBuilding(unittest.TestCase):

    def test_suite_data(self):
        suite = build('pass_and_fail.robot')
        assert_equal(suite.name, 'Pass And Fail')
        assert_equal(suite.doc, 'Some tests here')
        assert_equal(suite.metadata, {})

    def test_imports(self):
        imp = build('dummy_lib_test.robot').resource.imports[0]
        assert_equal(imp.type, 'LIBRARY')
        assert_equal(imp.name, 'DummyLib')
        assert_equal(imp.args, ())

    def test_variables(self):
        variables = build('pass_and_fail.robot').resource.variables
        assert_equal(variables[0].name, '${LEVEL1}')
        assert_equal(variables[0].value, ('INFO',))
        assert_equal(variables[1].name, '${LEVEL2}')
        assert_equal(variables[1].value, ('DEBUG',))

    def test_user_keywords(self):
        uk = build('pass_and_fail.robot').resource.keywords[0]
        assert_equal(uk.name, 'My Keyword')
        assert_equal(uk.args, ('${who}',))

    def test_test_data(self):
        test = build('pass_and_fail.robot').tests[1]
        assert_equal(test.name, 'Fail')
        assert_equal(test.doc, 'FAIL Expected failure')
        assert_equal(list(test.tags), ['fail', 'force'])
        assert_equal(test.timeout, None)
        assert_equal(test.template, None)

    def test_test_keywords(self):
        kw = build('pass_and_fail.robot').tests[0].body[0]
        assert_keyword(kw, (), 'My Keyword', ('Pass',))

    def test_assign(self):
        kw = build('non_ascii.robot').tests[1].body[0]
        assert_keyword(kw, ('${msg} =',), 'Evaluate', (r"u'Fran\\xe7ais'",))

    def test_directory_suite(self):
        suite = build('suites')
        assert_equal(suite.name, 'Suites')
        assert_equal(suite.suites[0].name, 'Suite With Prefix')
        assert_equal(suite.suites[2].name, 'Subsuites')
        assert_equal(suite.suites[4].name, 'Suite With Double Underscore')
        assert_equal(suite.suites[4].suites[0].name, 'Tests With Double Underscore')
        assert_equal(suite.suites[-1].name, 'Tsuite3')
        assert_equal(suite.suites[2].suites[1].name, 'Sub2')
        assert_equal(len(suite.suites[2].suites[1].tests), 1)
        assert_equal(suite.suites[2].suites[1].tests[0].id, 's1-s3-s2-t1')

    def test_multiple_inputs(self):
        suite = build('pass_and_fail.robot', 'normal.robot')
        assert_equal(suite.name, 'Pass And Fail & Normal')
        assert_equal(suite.suites[0].name, 'Pass And Fail')
        assert_equal(suite.suites[1].name, 'Normal')
        assert_equal(suite.suites[1].tests[1].id, 's1-s2-t2')

    def test_suite_setup_and_teardown(self):
        suite = build('setups_and_teardowns.robot')
        assert_keyword(suite.setup, name='${SUITE SETUP}', type='SETUP')
        assert_keyword(suite.teardown, name='${SUITE TEARDOWN}', type='TEARDOWN')

    def test_test_setup_and_teardown(self):
        test = build('setups_and_teardowns.robot').tests[0]
        assert_keyword(test.setup, name='${TEST SETUP}', type='SETUP')
        assert_keyword(test.teardown, name='${TEST TEARDOWN}', type='TEARDOWN')
        assert_equal([kw.name for kw in test.body], ['Keyword'])

    def test_test_timeout(self):
        tests = build('timeouts.robot').tests
        assert_equal(tests[0].timeout, '1min 42s')
        assert_equal(tests[1].timeout, '${100}')
        assert_equal(tests[2].timeout, None)

    def test_keyword_timeout(self):
        kw = build('timeouts.robot').resource.keywords[0]
        assert_equal(kw.timeout, '42')

    def test_rpa(self):
        for paths in [('.',), ('pass_and_fail.robot',),
                      ('pass_and_fail.robot', 'normal.robot')]:
            self._validate_rpa(build(*paths), False)
            self._validate_rpa(build(*paths, rpa=True), True)
        self._validate_rpa(build('../rpa/tasks1.robot'), True)
        self._validate_rpa(build('../rpa/', rpa=False), False)
        suite = build('../rpa/')
        assert_equal(suite.rpa, None)
        for child in suite.suites:
            self._validate_rpa(child, child.name != 'Tests')

    def _validate_rpa(self, suite, expected):
        assert_equal(suite.rpa, expected, suite.name)
        for child in suite.suites:
            self._validate_rpa(child, expected)

    def test_custom_parser(self):
        path = DATADIR / '../parsing/custom/CustomParser.py'
        for parser in [path, str(path)]:
            suite = build('../parsing/custom/tests.custom', custom_parsers=[parser])
            assert_equal(suite.name, 'Tests')
            assert_equal([t.name for t in suite.tests], ['Passing', 'Failing', 'Empty'])

    def test_custom_parser_with_args(self):
        path = DATADIR / '../parsing/custom/CustomParser.py:custom'
        for parser in [path, str(path)]:
            suite = build('../parsing/custom/tests.custom', custom_parsers=[parser])
            assert_equal(suite.name, 'Tests')
            assert_equal([t.name for t in suite.tests], ['Passing', 'Failing', 'Empty'])

    def test_custom_parser_as_object(self):
        path = DATADIR / '../parsing/custom/CustomParser.py'
        parser = Importer().import_class_or_module(path, instantiate_with_args=())
        suite = build('../parsing/custom/tests.custom', custom_parsers=[parser])
        assert_equal(suite.name, 'Tests')
        assert_equal([t.name for t in suite.tests], ['Passing', 'Failing', 'Empty'])

    def test_failing_parser_import(self):
        err = assert_raises(DataError, build, custom_parsers=['non_existing_mod'])
        assert_true(err.message.startswith("Importing parser 'non_existing_mod' failed:"))

    def test_incompatible_parser_object(self):
        err = assert_raises(DataError, build, custom_parsers=[42])
        assert_equal(err.message, "Importing parser 'integer' failed: "
                                  "'integer' does not have mandatory 'parse' method.")


class TestTemplates(unittest.TestCase):

    def test_from_setting_table(self):
        test = build('../running/test_template.robot').tests[0]
        assert_keyword(test.body[0], (), 'Should Be Equal', ('Fail', 'Fail'))
        assert_equal(test.template, 'Should Be Equal')

    def test_from_test_case(self):
        test = build('../running/test_template.robot').tests[3]
        kws = test.body
        assert_keyword(kws[0], (), 'Should Not Be Equal', ('Same', 'Same'))
        assert_keyword(kws[1], (), 'Should Not Be Equal', ('42', '43'))
        assert_keyword(kws[2], (), 'Should Not Be Equal', ('Something', 'Different'))
        assert_equal(test.template, 'Should Not Be Equal')

    def test_no_variable_assign(self):
        test = build('../running/test_template.robot').tests[8]
        assert_keyword(test.body[0], (), 'Expect Exactly Three Args',
                       ('${SAME VARIABLE}', 'Variable content', '${VARIABLE}'))
        assert_equal(test.template, 'Expect Exactly Three Args')


if __name__ == '__main__':
    unittest.main()



import sys

from remoteserver import RemoteServer


class DictResult:

    def return_dict(self, **kwargs):
        return kwargs

    def return_nested_dict(self):
        return dict(key='root', nested=dict(key=42, nested=dict(key='leaf')))

    def return_dict_in_list(self):
        return [{'foo': 1}, self.return_nested_dict()]


if __name__ == '__main__':
    RemoteServer(DictResult(), *sys.argv[1:])



from os.path import abspath, dirname, join
from fnmatch import fnmatchcase
from operator import eq

from robot.api import logger
from robot.api.deco import keyword


ROBOT_AUTO_KEYWORDS = False
CURDIR = dirname(abspath(__file__))


@keyword
def output_should_be(actual, expected, **replaced):
    actual = _read_file(actual, 'Actual')
    expected = _read_file(join(CURDIR, expected), 'Expected', replaced)
    if len(expected) != len(actual):
        raise AssertionError('Lengths differ. Expected %d lines but got %d'
                             % (len(expected), len(actual)))
    for exp, act in zip(expected, actual):
        tester = fnmatchcase if '*' in exp else eq
        if not tester(act.rstrip(), exp.rstrip()):
            raise AssertionError('Lines differ.\nExpected: %s\nActual:   %s'
                                 % (exp, act))


def _read_file(path, title, replaced=None):
    with open(path) as file:
        content = file.read()
    if replaced:
        for item in replaced:
            content = content.replace(item, replaced[item])
    logger.debug('%s:\n%s' % (title, content))
    return content.splitlines()



import sys
import unittest
from glob import glob
from io import StringIO
from os import remove
from os.path import exists

from robot.utils import is_integer


class RunningTestCase(unittest.TestCase):
    remove_files = []

    def setUp(self):
        self.orig__stdout__ = sys.__stdout__
        self.orig__stderr__ = sys.__stderr__
        self.orig_stdout = sys.stdout
        self.orig_stderr = sys.stderr
        self._setup_output_streams()
        self._remove_files()

    def tearDown(self):
        sys.__stdout__ = self.orig__stdout__
        sys.__stderr__ = self.orig__stderr__
        sys.stdout = self.orig_stdout
        sys.stderr = self.orig_stderr
        self._remove_files()

    def _setup_output_streams(self):
        sys.__stdout__ = StringIO()
        sys.__stderr__ = StringIO()
        sys.stdout = StringIO()
        sys.stderr = StringIO()

    def _clear_outputs(self):
        self._setup_output_streams()

    def _assert_outputs(self, stdout=None, stderr=None):
        self._assert_output(sys.__stdout__, stdout)
        self._assert_output(sys.__stderr__, stderr)
        self._assert_output(sys.stdout, None)
        self._assert_output(sys.stderr, None)

    def _assert_output(self, stream, expected):
        output = stream.getvalue()
        if expected:
            for content, count in expected:
                self._assert_output_contains(output, content, count)
        else:
            self._assert_no_output(output)

    def _assert_no_output(self, output):
        if output:
            raise AssertionError('Expected output to be empty:\n%s' % output)

    def _assert_output_contains(self, output, content, count):
        if is_integer(count):
            if output.count(content) != count:
                raise AssertionError("'%s' not %d times in output:\n%s"
                                     % (content, count, output))
        else:
            min_count, max_count = count
            if not (min_count <= output.count(content) <= max_count):
                raise AssertionError("'%s' not %d-%d times in output:\n%s"
                                     % (content, min_count,max_count, output))

    def _remove_files(self):
        for pattern in self.remove_files:
            for path in glob(pattern):
                if exists(path):
                    remove(path)



import os
import re

from xmlschema import XMLSchema

from robot import utils
from robot.api import logger
from robot.utils.asserts import assert_equal
from robot.result import (ExecutionResultBuilder, Result, ResultVisitor,
                          For, ForIteration, While, WhileIteration, Return, Break, Continue,
                          If, IfBranch, Try, TryBranch, Keyword, TestCase, TestSuite)
from robot.result.model import Body, Iterations
from robot.libraries.BuiltIn import BuiltIn


class NoSlotsKeyword(Keyword):
    pass


class NoSlotsFor(For):
    pass


class NoSlotsWhile(While):
    pass


class NoSlotsIf(If):
    pass


class NoSlotsTry(Try):
    pass


class NoSlotsReturn(Return):
    pass


class NoSlotsBreak(Break):
    pass


class NoSlotsContinue(Continue):
    pass


class NoSlotsBody(Body):
    keyword_class = NoSlotsKeyword
    for_class = NoSlotsFor
    if_class = NoSlotsIf
    try_class = NoSlotsTry
    while_class = NoSlotsWhile
    return_class = NoSlotsReturn
    break_class = NoSlotsBreak
    continue_class = NoSlotsContinue


class NoSlotsIfBranch(IfBranch):
    body_class = NoSlotsBody


class NoSlotsTryBranch(TryBranch):
    body_class = NoSlotsBody


class NoSlotsForIteration(ForIteration):
    body_class = NoSlotsBody


class NoSlotsWhileIteration(WhileIteration):
    body_class = NoSlotsBody


class NoSlotsIterations(Iterations):
    keyword_class = NoSlotsKeyword


NoSlotsKeyword.body_class = NoSlotsReturn.body_class = NoSlotsBreak.body_class \
    = NoSlotsContinue.body_class = NoSlotsBody
NoSlotsFor.iterations_class = NoSlotsWhile.iterations_class =NoSlotsIterations
NoSlotsFor.iteration_class = NoSlotsForIteration
NoSlotsWhile.iteration_class = NoSlotsWhileIteration
NoSlotsIf.branch_class = NoSlotsIfBranch
NoSlotsTry.branch_class = NoSlotsTryBranch


class NoSlotsTestCase(TestCase):
    fixture_class = NoSlotsKeyword
    body_class = NoSlotsBody


class NoSlotsTestSuite(TestSuite):
    fixture_class = NoSlotsKeyword
    test_class = NoSlotsTestCase


class TestCheckerLibrary:
    ROBOT_LIBRARY_SCOPE = 'GLOBAL'

    def __init__(self):
        self.schema = XMLSchema('doc/schema/robot.xsd')

    def process_output(self, path, validate=None):
        set_suite_variable = BuiltIn().set_suite_variable
        if not path or path.upper() == 'NONE':
            set_suite_variable('$SUITE', None)
            logger.info("Not processing output.")
            return
        path = path.replace('/', os.sep)
        if validate is None:
            validate = os.getenv('ATEST_VALIDATE_OUTPUT', False)
        if utils.is_truthy(validate):
            self._validate_output(path)
        try:
            logger.info("Processing output '%s'." % path)
            result = Result(root_suite=NoSlotsTestSuite())
            ExecutionResultBuilder(path).build(result)
        except:
            set_suite_variable('$SUITE', None)
            msg, details = utils.get_error_details()
            logger.info(details)
            raise RuntimeError('Processing output failed: %s' % msg)
        result.visit(ProcessResults())
        set_suite_variable('$SUITE', result.suite)
        set_suite_variable('$STATISTICS', result.statistics)
        set_suite_variable('$ERRORS', result.errors)

    def _validate_output(self, path):
        schema_version = self._get_schema_version(path)
        if schema_version != self.schema.version:
            raise AssertionError(
                'Incompatible schema versions. Schema has `version="%s"` '
                'but output file has `schemaversion="%s"`.'
                % (self.schema.version, schema_version)
        )
        self.schema.validate(path)

    def _get_schema_version(self, path):
        with open(path, encoding='UTF-8') as f:
            for line in f:
                if line.startswith('<robot'):
                    return re.search(r'schemaversion="(\d+)"', line).group(1)

    def get_test_case(self, name):
        suite = BuiltIn().get_variable_value('${SUITE}')
        return self._get_test_from_suite(suite, name)

    def _get_test_from_suite(self, suite, name):
        tests = self.get_tests_from_suite(suite, name)
        if len(tests) == 1:
            return tests[0]
        err = "No test '%s' found from suite '%s'" if not tests \
            else "More than one test '%s' found from suite '%s'"
        raise RuntimeError(err % (name, suite.name))

    def get_tests_from_suite(self, suite, name=None):
        tests = [test for test in suite.tests
                 if name is None or utils.eq(test.name, name)]
        for subsuite in suite.suites:
            tests.extend(self.get_tests_from_suite(subsuite, name))
        return tests

    def get_test_suite(self, name):
        suite = BuiltIn().get_variable_value('${SUITE}')
        suites = self._get_suites_from_suite(suite, name)
        if len(suites) == 1:
            return suites[0]
        err = "No suite '%s' found from suite '%s'" if not suites \
            else "More than one suite '%s' found from suite '%s'"
        raise RuntimeError(err % (name, suite.name))

    def _get_suites_from_suite(self, suite, name):
        suites = [suite] if utils.eq(suite.name, name) else []
        for subsuite in suite.suites:
            suites.extend(self._get_suites_from_suite(subsuite, name))
        return suites

    def check_test_case(self, testname, status=None, message=None):
        test = self._get_test_from_suite(BuiltIn().get_variable_value('${SUITE}'), testname)
        self._check_test_status(test, status=status, message=message)
        return test

    def _check_test_status(self, test, status=None, message=None):
        """Verifies that test's status and message are as expected.

        Expected status and message can be given as parameters. If expected
        status is not given, expected status and message are read from test's
        documentation. If documentation doesn't contain any of PASS, FAIL or
        ERROR, test's status is expected to be PASS. If status is given that is
        used. Expected message is documentation after given status. Expected
        message can also be regular expression. In that case expected match
        starts with REGEXP: , which is ignored in the regexp match.
        """
        if status is not None:
            test.exp_status = status
        if message is not None:
            test.exp_message = message
        if test.exp_status != test.status:
            if test.exp_status == 'PASS':
                if test.status == 'FAIL':
                    msg = f"Error message:\n{test.message}"
                else:
                    msg = f"Test message:\n{test.message}"
            else:
                msg = f"Expected message:\n{test.exp_message}"
            raise AssertionError(
                f"Status of '{test.name}' should have been {test.exp_status} "
                f"but it was {test.status}.\n\n{msg}"
            )
        if test.exp_message == test.message:
            return
        if test.exp_message.startswith('REGEXP:'):
            pattern = self._get_pattern(test, 'REGEXP:')
            if re.match('^%s$' % pattern, test.message, re.DOTALL):
                return
        if test.exp_message.startswith('GLOB:'):
            pattern = self._get_pattern(test, 'GLOB:')
            matcher = utils.Matcher(pattern, caseless=False, spaceless=False)
            if matcher.match(test.message):
                return
        if test.exp_message.startswith('STARTS:'):
            start = self._get_pattern(test, 'STARTS:')
            if test.message.startswith(start):
                return
        raise AssertionError("Test '%s' had wrong message.\n\n"
                             "Expected:\n%s\n\nActual:\n%s\n"
                             % (test.name, test.exp_message, test.message))

    def _get_pattern(self, test, prefix):
        pattern = test.exp_message[len(prefix):].strip()
        if not pattern:
            raise RuntimeError("Empty '%s' is not allowed!")
        return pattern

    def should_contain_tests(self, suite, *names, **names_and_statuses):
        """Verify that specified tests exists in suite.

        'names' contains test names to check. These tests are expected to
        pass/fail as their documentation says. Is same name is given multiple
        times, test ought to be executed multiple times too.

        'names_and_statuses' contains test names with associated custom status
        in format `STATUS:Message`. Test is given both in 'names' and in
        'names_and_statuses', only the latter has an effect.
        """
        tests = self.get_tests_from_suite(suite)
        expected = [(n, None) for n in names if n not in names_and_statuses]
        expected.extend((n, s) for n, s in names_and_statuses.items())
        tests_msg = "\nExpected tests : %s\nActual tests   : %s" \
                    % (', '.join(sorted([e[0] for e in expected], key=lambda s: s.lower())),
                       ', '.join(sorted([t.name for t in tests], key=lambda s: s.lower())))
        if len(tests) != len(expected):
            raise AssertionError("Wrong number of tests." + tests_msg)
        for test in tests:
            logger.info("Verifying test '%s'" % test.name)
            try:
                status = self._find_expected_status(test.name, expected)
            except IndexError:
                raise AssertionError("Test '%s' was not expected to be run.%s"
                                     % (test.name, tests_msg))
            expected.pop(expected.index((test.name, status)))
            if status and ':' in status:
                status, message = status.split(':', 1)
            else:
                message = None
            self._check_test_status(test, status, message)
        assert not expected

    def _find_expected_status(self, test, expected):
        for name, status in expected:
            if name == test:
                return status
        raise IndexError

    def should_not_contain_tests(self, suite, *test_names):
        actual_names = [t.name for t in suite.tests]
        for name in test_names:
            if name in actual_names:
                raise AssertionError('Suite should not have contained test "%s"' % name)

    def should_contain_suites(self, suite, *expected):
        logger.info('Suite has suites', suite.suites)
        expected = sorted(expected)
        actual = sorted(s.name for s in suite.suites)
        if len(actual) != len(expected):
            raise AssertionError("Wrong number of suites.\n"
                                 "Expected (%d): %s\n"
                                 "Actual   (%d): %s"
                                 % (len(expected), ', '.join(expected),
                                    len(actual), ', '.join(actual)))
        for name in expected:
            if not utils.Matcher(name).match_any(actual):
                raise AssertionError('Suite %s not found' % name)

    def should_contain_tags(self, test, *tags):
        logger.info('Test has tags', test.tags)
        assert_equal(len(test.tags), len(tags), 'Wrong number of tags')
        tags = sorted(tags, key=lambda s: s.lower().replace('_', '').replace(' ', ''))
        for act, exp in zip(test.tags, tags):
            assert_equal(act, exp)

    def should_contain_keywords(self, item, *kw_names):
        actual_names = [kw.name for kw in item.kws]
        assert_equal(len(actual_names), len(kw_names), 'Wrong number of keywords')
        for act, exp in zip(actual_names, kw_names):
            assert_equal(act, exp)

    def test_should_have_correct_keywords(self, *kw_names, **config):
        get_var = BuiltIn().get_variable_value
        suite = get_var('${SUITE}')
        name = config.get('name', get_var('${TEST NAME}'))
        kw_index = int(config.get('kw_index', 0))
        test = self._get_test_from_suite(suite, name)
        self._check_test_status(test)
        self.should_contain_keywords(test.body[kw_index], *kw_names)
        return test

    def check_log_message(self, item, expected, level='INFO', html=False, pattern=False, traceback=False):
        message = item.message.rstrip()
        if traceback:
            # Remove `^^^` lines added by Python 3.11+.
            message = '\n'.join(line for line in message.splitlines()
                                if '^' not in line or line.strip('^ '))
        b = BuiltIn()
        matcher = b.should_match if pattern else b.should_be_equal
        matcher(message, expected.rstrip(), 'Wrong log message')
        if level != 'IGNORE':
            b.should_be_equal(item.level, 'INFO' if level == 'HTML' else level, 'Wrong log level')
        b.should_be_equal(str(item.html), str(html or level == 'HTML'), 'Wrong HTML status')


class ProcessResults(ResultVisitor):

    def start_test(self, test):
        for status in 'FAIL', 'SKIP', 'PASS':
            if status in test.doc:
                test.exp_status = status
                test.exp_message = test.doc.split(status, 1)[1].lstrip()
                break
        else:
            test.exp_status = 'PASS'
            test.exp_message = ''
        test.kws = list(test.body)
        test.keyword_count = test.kw_count = len(test.kws)

    def start_keyword(self, kw):
        self._add_kws_and_msgs(kw)

    def _add_kws_and_msgs(self, item):
        # TODO: Consider not setting these special attributes:
        # - Using normal `body` instead of special `kws` in tests would be better.
        # - `msgs` isn't that much shorter than normal `messages`.
        # - Counts likely not needed often enough. There are other ways to get them.
        # - No need to construct "NoSlots" variants for all model objects.
        item.kws = item.body.filter(messages=False)
        item.msgs = item.body.filter(messages=True)
        item.keyword_count = item.kw_count = len(item.kws)
        item.message_count = item.msg_count = len(item.msgs)

    def start_for(self, for_):
        self._add_kws_and_msgs(for_)

    def start_for_iteration(self, iteration):
        self._add_kws_and_msgs(iteration)

    def start_if(self, if_):
        self._add_kws_and_msgs(if_)

    def start_if_branch(self, branch):
        self._add_kws_and_msgs(branch)

    def start_while(self, while_):
        self._add_kws_and_msgs(while_)

    def start_while_iteration(self, iteration):
        self._add_kws_and_msgs(iteration)

    def visit_error(self, error):
        pass

    def visit_errors(self, errors):
        errors.msgs = errors.messages
        errors.message_count = errors.msg_count = len(errors.messages)



import sys

from robot.libraries.BuiltIn import BuiltIn


class global_vars_listenerlibrary():
    ROBOT_LISTENER_API_VERSION = 2

    global_vars = ["${SUITE_NAME}",
                   "${SUITE_DOCUMENTATION}",
                   "${PREV_TEST_NAME}",
                   "${PREV_TEST_STATUS}",
                   "${LOG_LEVEL}"]

    def __init__(self):
        self.ROBOT_LIBRARY_LISTENER = self

    def _close(self):
        for var in self.global_vars:
            sys.__stderr__.write('%s: %s\n' % (var, BuiltIn().get_variable_value(var)))



class DynamicLibraryWithoutArgspec:

    def get_keyword_names(self):
        return [name for name in dir(self) if name.startswith('do_')]

    def run_keyword(self, name, args):
        return getattr(self, name)(*args)

    def do_something(self, x):
        print(x)

    def do_something_else(self, x, y=0):
        print('x: %s, y: %s' % (x, y))

    def do_something_third(self, a, b=2, c=3):
        print(a, b, c)



from robot.api.deco import keyword


@keyword(name=r'${num1:\d+} + ${num2:\d+} = ${exp:\d+}')
def add(num1: int, num2: int, expected: int):
    result = num1 + num2
    assert result == expected, (result, expected)


@keyword(name=r'${num1:\d+} - ${num2:\d+} = ${exp:\d+}', types=(int, int, int))
def sub(num1, num2, expected):
    result = num1 - num2
    assert result == expected, (result, expected)


@keyword(name=r'${num1:\d+} * ${num2:\d+} = ${exp:\d+}')
def mul(num1=0, num2=0, expected=0):
    result = num1 * num2
    assert result == expected, (result, expected)



from datetime import date, datetime
from typing import Dict, List, Set, Tuple, Union
from types import ModuleType
try:
    from typing import TypedDict
except ImportError:
    from typing_extensions import TypedDict

from robot.api.deco import not_keyword


not_keyword(TypedDict)


class Number:
    pass


def string_to_int(value: str) -> int:
    try:
        return ['zero', 'one', 'two', 'three', 'four'].index(value.lower())
    except ValueError:
        raise ValueError(f"Don't know number {value!r}.")


class String:
    pass


def int_to_string_with_lib(value: int, library) -> str:
    if library is None:
        raise AssertionError('Expected library, got none')
    if not isinstance(library, ModuleType):
        raise AssertionError(f'Expected library to be instance of {ModuleType}, was {type(library)}')
    return str(value)


def parse_bool(value: Union[str, int, bool]):
    if isinstance(value, str):
        value = value.lower()
    return value not in ['false', '', 'epätosi', '\u2639', False, 0]


class UsDate(date):
    @classmethod
    def from_string(cls, value) -> date:
        if not isinstance(value, str):
            raise TypeError("Only strings accepted!")
        try:
            return cls.fromordinal(datetime.strptime(value, '%m/%d/%Y').toordinal())
        except ValueError:
            raise ValueError("Value does not match '%m/%d/%Y'.")


class FiDate(date):
    @classmethod
    def from_string(cls, value: str, ign1=None, *ign2, ign3=None, **ign4):
        try:
            return cls.fromordinal(datetime.strptime(value, '%d.%m.%Y').toordinal())
        except ValueError:
            raise RuntimeError("Value does not match '%d.%m.%Y'.")


class ClassAsConverter:
    def __init__(self, name):
        self.greeting = f'Hello, {name}!'


class ClassWithHintsAsConverter:
    name: str

    def __init__(self, value: Union[int, str]):
        self.value = value


class AcceptSubscriptedGenerics:
    def __init__(self, numbers: List[int]):
        self.sum = sum(numbers)


class OnlyVarArg:
    def __init__(self, *varargs):
        self.value = varargs[0]
        library = varargs[1]
        if library is None:
            raise AssertionError('Expected library, got none')
        if not isinstance(library, ModuleType):
            raise AssertionError(f'Expected library to be instance of {ModuleType}, was {type(library)}')



class Strict:
    pass


class Invalid:
    pass


class TooFewArgs:
    pass


class TooManyArgs:
    def __init__(self, one, two, three):
        pass


class NoPositionalArg:
    def __init__(self, *, args):
        pass


class KwOnlyNotOk:
    def __init__(self, arg, *, kwo, another):
        pass


ROBOT_LIBRARY_CONVERTERS = {Number: string_to_int,
                            bool: parse_bool,
                            String: int_to_string_with_lib,
                            UsDate: UsDate.from_string,
                            FiDate: FiDate.from_string,
                            ClassAsConverter: ClassAsConverter,
                            ClassWithHintsAsConverter: ClassWithHintsAsConverter,
                            AcceptSubscriptedGenerics: AcceptSubscriptedGenerics,
                            OnlyVarArg: OnlyVarArg,
                            Strict: None,
                            Invalid: 666,
                            TooFewArgs: TooFewArgs,
                            TooManyArgs: TooManyArgs,
                            NoPositionalArg: NoPositionalArg,
                            KwOnlyNotOk: KwOnlyNotOk,
                            'Bad': int}


def only_var_arg(argument: OnlyVarArg, expected):
    assert isinstance(argument, OnlyVarArg)
    assert argument.value == expected


def number(argument: Number, expected: int = 0):
    if argument != expected:
        raise AssertionError(f'Expected value to be {expected!r}, got {argument!r}.')


def true(argument: bool):
    assert argument is True


def false(argument: bool):
    assert argument is False


def string(argument: String, expected: str = '123'):
    if argument != expected:
        raise AssertionError


def us_date(argument: UsDate, expected: date = None):
    assert argument == expected


def fi_date(argument: FiDate, expected: date = None):
    assert argument == expected


def dates(us: 'UsDate', fi: 'FiDate'):
    assert us == fi


def class_as_converter(argument: ClassAsConverter, expected):
    assert argument.greeting == expected


def class_with_hints_as_converter(argument: ClassWithHintsAsConverter, expected=None):
    assert argument.value == expected


def accept_subscripted_generics(argument: AcceptSubscriptedGenerics, expected):
    assert argument.sum == expected


def with_generics(a: List[Number], b: Tuple[FiDate, UsDate], c: Dict[Number, FiDate], d: Set[Number]):
    expected_date = date(2022, 9, 28)
    assert a == [1, 2, 3], a
    assert b == (expected_date, expected_date), b
    assert c == {1: expected_date}, c
    assert d == {1, 2, 3}, d


def typeddict(dates: TypedDict('Dates', {'fi': FiDate, 'us': UsDate})):
    fi, us = dates['fi'], dates['us']
    exp = date(2022, 9, 29)
    assert isinstance(fi, FiDate) and isinstance(us, UsDate) and fi == us == exp


def number_or_int(number: Union[Number, int]):
    assert number == 1


def int_or_number(number: Union[int, Number]):
    assert number == 1


def strict(argument: Strict):
    assert isinstance(argument, Strict)


def invalid(a: Invalid, b: TooFewArgs, c: TooManyArgs, d: KwOnlyNotOk):
    assert (a, b, c, d) == ('a', 'b', 'c', 'd')


def non_type_annotation(arg1: 'Hello, world!', arg2: 2 = 2):
    assert arg1 == arg2


def multiplying_converter(value: str, library) -> int:
    return library.counter * int(value)


class StatefulLibrary:
    ROBOT_LIBRARY_CONVERTERS = {Number: multiplying_converter}

    def __init__(self):
        self.counter = 1

    def multiply(self, num: Number, expected: int):
        self.counter += 1
        assert num == int(expected)


class StatefulGlobalLibrary:
    ROBOT_LIBRARY_SCOPE = 'GLOBAL'
    ROBOT_LIBRARY_CONVERTERS = {Number: multiplying_converter}

    def __init__(self):
        self.counter = 1

    def global_multiply(self, num: Number, expected: int):
        self.counter += 1
        assert num == int(expected)



from robot.libraries.BuiltIn import BuiltIn

class NamespaceUsingLibrary:

    def __init__(self):
        self._importing_suite = BuiltIn().get_variable_value('${SUITE NAME}')
        self._easter = BuiltIn().get_library_instance('Easter')

    def get_importing_suite(self):
        return self._importing_suite

    def get_other_library(self):
        return self._easter



from collections import abc
from datetime import datetime, date, timedelta
from decimal import Decimal
from enum import Flag, Enum, IntFlag, IntEnum
from functools import wraps
from numbers import Integral, Real
from os import PathLike
from pathlib import Path, PurePath

# Needed by `eval()` in `_validate_type()`.
import collections
from fractions import Fraction

from robot.api.deco import keyword


class MyEnum(Enum):
    FOO = 1
    bar = 'xxx'
    foo = 'yyy'
    normalize_me = True


class NoneEnum(Enum):
    NONE = 1
    NTWO = 2
    NTHREE = 3


class MyFlag(Flag):
    RED = 1
    BLUE = 2


class MyIntEnum(IntEnum):
    ON = 1
    OFF = 0


class MyIntFlag(IntFlag):
    R = 4
    W = 2
    X = 1


class Unknown:
    pass


def integer(argument: int, expected=None):
    _validate_type(argument, expected)


def integral(argument: Integral, expected=None):
    _validate_type(argument, expected)


def float_(argument: float, expected=None):
    _validate_type(argument, expected)


def real(argument: Real, expected=None):
    _validate_type(argument, expected)


def decimal(argument: Decimal, expected=None):
    _validate_type(argument, expected)


def boolean(argument: bool, expected=None):
    _validate_type(argument, expected)


def string(argument: str, expected=None):
    _validate_type(argument, expected)


def bytes_(argument: bytes, expected=None):
    _validate_type(argument, expected)


def bytestring(argument: abc.ByteString, expected=None):
    _validate_type(argument, expected)


def bytearray_(argument: bytearray, expected=None):
    _validate_type(argument, expected)


def datetime_(argument: datetime, expected=None):
    _validate_type(argument, expected)


def date_(argument: date, expected=None):
    _validate_type(argument, expected)


def timedelta_(argument: timedelta, expected=None):
    _validate_type(argument, expected)


def path(argument: Path, expected=None):
    _validate_type(argument, expected)


def pure_path(argument: PurePath, expected=None):
    _validate_type(argument, expected)


def path_like(argument: PathLike, expected=None):
    _validate_type(argument, expected)


def enum_(argument: MyEnum, expected=None):
    _validate_type(argument, expected)


def none_enum(argument: NoneEnum, expected=None):
    _validate_type(argument, expected)


def flag(argument: MyFlag, expected=None):
    _validate_type(argument, expected)


def int_enum(argument: MyIntEnum, expected=None):
    _validate_type(argument, expected)


def int_flag(argument: MyIntFlag, expected=None):
    _validate_type(argument, expected)


def nonetype(argument: type(None), expected=None):
    _validate_type(argument, expected)


def list_(argument: list, expected=None):
    _validate_type(argument, expected)


def sequence(argument: abc.Sequence, expected=None):
    _validate_type(argument, expected)


def mutable_sequence(argument: abc.MutableSequence, expected=None):
    _validate_type(argument, expected)


def tuple_(argument: tuple, expected=None):
    _validate_type(argument, expected)


def dictionary(argument: dict, expected=None):
    _validate_type(argument, expected)


def mapping(argument: abc.Mapping, expected=None):
    _validate_type(argument, expected)


def mutable_mapping(argument: abc.MutableMapping, expected=None):
    _validate_type(argument, expected)


def set_(argument: set, expected=None):
    _validate_type(argument, expected)


def set_abc(argument: abc.Set, expected=None):
    _validate_type(argument, expected)


def mutable_set(argument: abc.MutableSet, expected=None):
    _validate_type(argument, expected)


def frozenset_(argument: frozenset, expected=None):
    _validate_type(argument, expected)


def unknown(argument: Unknown, expected=None):
    _validate_type(argument, expected)


def non_type(argument: 'this is string, not type', expected=None):
    _validate_type(argument, expected)


def unhashable(argument: {}, expected=None):
    _validate_type(argument, expected)


# Causes SyntaxError with `typing.get_type_hints`
def invalid(argument: 'import sys', expected=None):
    _validate_type(argument, expected)


def varargs(*argument: int, expected=None):
    _validate_type(argument, expected)


def kwargs(expected=None, **argument: int):
    _validate_type(argument, expected)


def kwonly(*, argument: float, expected=None):
    _validate_type(argument, expected)


def none_as_default(argument: list = None, expected=None):
    _validate_type(argument, expected)


def none_as_default_with_unknown_type(argument: Unknown = None, expected=None):
    _validate_type(argument, expected)


def forward_referenced_concrete_type(argument: 'int', expected=None):
    _validate_type(argument, expected)


def forward_referenced_abc(argument: 'abc.Sequence', expected=None):
    _validate_type(argument, expected)


def return_value_annotation(argument: int, expected=None) -> float:
    _validate_type(argument, expected)
    return float(argument)


@keyword(types={'argument': timedelta})
def types_via_keyword_deco_override(argument: int, expected=None):
    _validate_type(argument, expected)


@keyword(name='None as types via @keyword disables', types=None)
def none_as_types(argument: int, expected=None):
    _validate_type(argument, expected)


@keyword(name="Empty types via @keyword doesn't override", types=[])
def empty_list_as_types(argument: int, expected=None):
    _validate_type(argument, expected)


@keyword(name="@keyword without types doesn't override")
def keyword_deco_alone_does_not_override(argument: int, expected=None):
    _validate_type(argument, expected)


def decorator(func):
    def wrapper(*args, **kws):
        return func(*args, **kws)
    return wrapper


def decorator_with_wraps(func):
    @wraps(func)
    def wrapper(*args, **kws):
        return func(*args, **kws)
    return wrapper


@decorator
def mismatch_caused_by_decorator(argument: int, expected=None):
    _validate_type(argument, expected)


@decorator_with_wraps
def keyword_with_wraps(argument: int, expected=None):
    _validate_type(argument, expected)


def type_and_default_1(argument: list = None, expected=None):
    _validate_type(argument, expected)


def type_and_default_2(argument: int = True, expected=None):
    _validate_type(argument, expected)


def type_and_default_3(argument: timedelta = 0, expected=None):
    _validate_type(argument, expected)


def type_and_default_4(argument: list = [], expected=None):
    _validate_type(argument, expected)


def _validate_type(argument, expected):
    if isinstance(expected, str):
        expected = eval(expected)
    if argument != expected or type(argument) != type(expected):
        raise AssertionError('%r (%s) != %r (%s)'
                             % (argument, type(argument).__name__,
                                expected, type(expected).__name__))



def annotations(arg1, arg2: str):
    return ' '.join(['annotations:', arg1, arg2])


def annotations_with_defaults(arg1, arg2: 'has a default' = 'default'):
    return ' '.join(['annotations:', arg1, arg2])



#!/usr/bin/env python

"""Script to generate atest runners based on plain text data files.

Usage:  {tool} testdata/path/data.robot [robot/path/runner.robot]
"""

from os.path import abspath, basename, dirname, exists, join
import os
import sys
import re

if len(sys.argv) not in [2, 3] or not all(a.endswith('.robot') for a in sys.argv[1:]):
    sys.exit(__doc__.format(tool=basename(sys.argv[0])))

SEPARATOR = re.compile(r'\s{2,}|\t')
INPATH = abspath(sys.argv[1])
if join('atest', 'testdata') not in INPATH:
    sys.exit("Input not under 'atest/testdata'.")
if len(sys.argv) == 2:
    OUTPATH = INPATH.replace(join('atest', 'testdata'), join('atest', 'robot'))
else:
    OUTPATH = sys.argv[2]

if not exists(dirname(OUTPATH)):
    os.mkdir(dirname(OUTPATH))


class TestCase:

    def __init__(self, name, tags=None):
        self.name = name
        self.tags = tags


with open(INPATH) as input:
    TESTS = []
    SETTINGS = []
    parsing_tests = False
    parsing_settings = False
    for line in input.readlines():
        line = line.rstrip()
        if not line:
            continue
        elif line.startswith('*'):
            name = SEPARATOR.split(line)[0].replace('*', '').replace(' ', '').upper()
            parsing_tests = name in ('TESTCASE', 'TESTCASES', 'TASK', 'TASKS')
            parsing_settings = name in ('SETTING', 'SETTINGS')
        elif parsing_tests and not SEPARATOR.match(line):
            TESTS.append(TestCase(line.split('  ')[0]))
        elif parsing_tests and line.strip().startswith('[Tags]'):
            TESTS[-1].tags = line.split('[Tags]', 1)[1].split()
        elif parsing_settings and line.startswith(('Force Tags', 'Default Tags', 'Test Tags')):
            name, value = line.split('  ', 1)
            SETTINGS.append((name, value.strip()))


with open(OUTPATH, 'w') as output:
    path = INPATH.split(join('atest', 'testdata'))[1][1:].replace(os.sep, '/')
    output.write('''\
*** Settings ***
Suite Setup       Run Tests    ${EMPTY}    %s
''' % path)
    for name, value in SETTINGS:
        output.write('%s%s\n' % (name.ljust(18), value))
    output.write('''\
Resource          atest_resource.robot

*** Test Cases ***
''')
    for test in TESTS:
        output.write(test.name + '\n')
        if test.tags:
            output.write('    [Tags]    %s\n' % '    '.join(test.tags))
        output.write('    Check Test Case    ${TESTNAME}\n')
        if test is not TESTS[-1]:
            output.write('\n')


print(OUTPATH)



from robot.api.deco import keyword


def library_keyword_tags_with_attribute():
    pass

library_keyword_tags_with_attribute.robot_tags = ['first', 'second']


@keyword(tags=('one', 2, '2', ''))
def library_keyword_tags_with_decorator():
    pass


def library_keyword_tags_with_documentation():
    """Summary line

    Tags: are read only from the last line

    Tags: one, two words"""
    pass


@keyword(tags=['one', 2])
def library_keyword_tags_with_documentation_and_attribute():
    """Tags: one, two words"""
    pass


@keyword(tags=42)
def invalid_library_keyword_tags():
    pass



# None of these decorators should be exposed as keywords.
from robot.api.deco import keyword, library, not_keyword

from os.path import abspath


not_keyword(abspath)


def exposed_in_module():
    pass


@not_keyword
def not_exposed_in_module():
    pass


@keyword
@not_keyword
def keyword_and_not_keyword():
    pass


def not_exposed_by_setting_attribute():
    pass


not_exposed_by_setting_attribute.robot_not_keyword = True



class no_arg_init:
    """No inits here!"""

    def __init__(self):
        """This doc not shown because there are no arguments."""

    def keyword(self, arg1, arg2):
        """The only lonely keyword."""



from robot.libraries.BuiltIn import BuiltIn


def log_debug_message():
    b = BuiltIn()
    b.set_log_level('DEBUG')
    b.log('Hello, debug world!', 'DEBUG')


def get_test_name():
    return BuiltIn().get_variables()['${TEST NAME}']


def set_secret_variable():
    BuiltIn().set_test_variable('${SECRET}', '*****')


def use_run_keyword_with_non_unicode_values():
    BuiltIn().run_keyword('Log', 42)
    BuiltIn().run_keyword('Log', b'\xff')


def user_keyword_via_run_keyword():
    BuiltIn().run_keyword("UseBuiltInResource.Keyword", 'This is x', 911)



from robot.api.deco import keyword


def passing_handler(*args):
    for arg in args:
        print(arg, end=' ')
    return ', '.join(args)


def failing_handler(*args):
    raise AssertionError('Failure: %s' % ' '.join(args) if args else 'Failure')


class GetKeywordNamesLibrary:

    def __init__(self):
        self.not_method_or_function = 'This is just a string!!'

    def get_keyword_names(self):
        marked = [name for name in dir(self)
                  if hasattr(getattr(self, name), 'robot_name')]
        other = ['Get Keyword That Passes', 'Get Keyword That Fails',
                 'keyword_in_library_itself', '_starting_with_underscore_is_ok',
                 'Non-existing attribute', 'not_method_or_function',
                 'Unexpected error getting attribute', '__init__']
        return marked + other

    def __getattr__(self, name):
        if name == 'Get Keyword That Passes':
            return passing_handler
        if name == 'Get Keyword That Fails':
            return failing_handler
        if name == 'Unexpected error getting attribute':
            raise TypeError('Oooops!')
        raise AttributeError("Non-existing attribute '%s'" % name)

    def keyword_in_library_itself(self):
        msg = 'No need for __getattr__ here!!'
        print(msg)
        return msg

    def _starting_with_underscore_is_ok(self):
        print("This is explicitly returned from 'get_keyword_names' anyway.")

    @keyword("Name set using 'robot_name' attribute")
    def name_set_in_method_signature(self):
        pass

    @keyword
    def keyword_name_should_not_change(self):
        pass

    @keyword('Add ${count} copies of ${item} to cart')
    def add_copies_to_cart(self, count, item):
        return count, item



import time


def timezone_correction():
    dst = 3600 if time.daylight == 0 else 0
    tz = 7200 + time.timezone
    return (tz + dst) * 1000

def timestamp_as_integer():
    t = 1308419034931 + timezone_correction()
    print('*INFO:%d* Known timestamp' % t)
    print('*HTML:%d* <b>Current</b>' % int(time.time() * 1000))
    time.sleep(0.1)

def timestamp_as_float():
    t = 1308419034930.502342313 + timezone_correction()
    print('*INFO:%f* Known timestamp' % t)
    print('*HTML:%f* <b>Current</b>' % float(time.time() * 1000))
    time.sleep(0.1)



import unittest

from robot.libdocpkg.standardtypes import STANDARD_TYPE_DOCS
from robot.running.arguments.typeconverters import (
    EnumConverter, CombinedConverter, CustomConverter, TypeConverter, TypedDictConverter
)


class TestStandardTypeDocs(unittest.TestCase):
    no_std_docs = (EnumConverter, CombinedConverter, CustomConverter, TypedDictConverter)

    def test_all_standard_types_have_docs(self):
        for cls in TypeConverter.__subclasses__():
            if cls.type not in STANDARD_TYPE_DOCS and cls not in self.no_std_docs:
                raise AssertionError(f"Standard converter '{cls.__name__}' "
                                     f"does not have documentation.")


if __name__ == '__main__':
    unittest.main()



from numbers import Rational
from typing import TypedDict


class MyObject:
    pass


class AnotherObject:
    pass


class BadRationalMeta(type(Rational)):
    def __instancecheck__(self, instance):
        raise TypeError('Bang!')


class BadRational(Rational, metaclass=BadRationalMeta):
    pass


def create_my_object():
    return MyObject()


def union_of_int_float_and_string(argument: int | float | str, expected):
    assert argument == expected


def union_of_int_and_float(argument: int | float, expected=object()):
    assert argument == expected


def union_with_int_and_none(argument: int | None, expected=object()):
    assert argument == expected


def union_with_int_none_and_str(argument: int | None | str, expected):
    assert argument == expected


def union_with_abc(argument: Rational | None, expected):
    assert argument == expected


def union_with_str_and_abc(argument: str | Rational, expected):
    assert argument == expected


def union_with_subscripted_generics(argument: list[int] | int, expected=object()):
    assert argument == eval(expected), '%r != %s' % (argument, expected)


def union_with_subscripted_generics_and_str(argument: list[str] | str, expected):
    assert argument == eval(expected), '%r != %s' % (argument, expected)


def union_with_typeddict(argument: TypedDict('X', x=int) | None, expected):
    assert argument == eval(expected), '%r != %s' % (argument, expected)


def union_with_item_not_liking_isinstance(argument: BadRational | bool, expected):
    assert argument == expected, '%r != %r' % (argument, expected)


def custom_type_in_union(argument: MyObject | str, expected_type):
    assert type(argument).__name__ == expected_type


def only_custom_types_in_union(argument: MyObject | AnotherObject, expected_type):
    assert type(argument).__name__ == expected_type


def union_with_string_first(argument: str | None, expected):
    assert argument == expected



MESSAGES = ['Circle is 360°',
            'Hyvää üötä',
            '\u0989\u09C4 \u09F0 \u09FA \u099F \u09EB \u09EA \u09B9']


class NonAsciiLibrary:

    def print_non_ascii_strings(self):
        """Prints message containing non-ASCII characters"""
        for msg in MESSAGES:
            print('*INFO*' + msg)

    def print_and_return_non_ascii_object(self):
        """Prints object with non-ASCII `str()` and returns it."""
        obj = NonAsciiObject()
        print(obj)
        return obj

    def raise_non_ascii_error(self):
        raise AssertionError(', '.join(MESSAGES))


class NonAsciiObject:

    def __init__(self):
        self.message = ', '.join(MESSAGES)

    def __str__(self):
        return self.message

    def __repr__(self):
        return repr(self.message)



class PythonClass:
    python_string = 'hello'
    python_integer = None
    LIST__python_list = ['a', 'b', 'c']

    def __init__(self):
        self.python_integer = 42

    def python_method(self):
        pass

    @property
    def python_property(self):
        return 'value'



# Imports needed for evaluating expected result.
from datetime import datetime, date, timedelta
from decimal import Decimal


def integer(argument: 'Integer', expected=None):
    _validate_type(argument, expected)


def int_(argument: 'INT', expected=None):
    _validate_type(argument, expected)


def long_(argument: 'lOnG', expected=None):
    _validate_type(argument, expected)


def float_(argument: 'Float', expected=None):
    _validate_type(argument, expected)


def double(argument: 'Double', expected=None):
    _validate_type(argument, expected)


def decimal(argument: 'DECIMAL', expected=None):
    _validate_type(argument, expected)


def boolean(argument: 'Boolean', expected=None):
    _validate_type(argument, expected)


def bool_(argument: 'Bool', expected=None):
    _validate_type(argument, expected)


def string(argument: 'String', expected=None):
    _validate_type(argument, expected)


def bytes_(argument: 'BYTES', expected=None):
    _validate_type(argument, expected)


def bytearray_(argument: 'ByteArray', expected=None):
    _validate_type(argument, expected)


def datetime_(argument: 'DateTime', expected=None):
    _validate_type(argument, expected)


def date_(argument: 'Date', expected=None):
    _validate_type(argument, expected)


def timedelta_(argument: 'TimeDelta', expected=None):
    _validate_type(argument, expected)


def list_(argument: 'List', expected=None):
    _validate_type(argument, expected)


def tuple_(argument: 'TUPLE', expected=None):
    _validate_type(argument, expected)


def dictionary(argument: 'Dictionary', expected=None):
    _validate_type(argument, expected)


def dict_(argument: 'Dict', expected=None):
    _validate_type(argument, expected)


def map_(argument: 'Map', expected=None):
    _validate_type(argument, expected)


def set_(argument: 'Set', expected=None):
    _validate_type(argument, expected)


def frozenset_(argument: 'FrozenSet', expected=None):
    _validate_type(argument, expected)


def _validate_type(argument, expected):
    if isinstance(expected, str):
        expected = eval(expected)
    if argument != expected or type(argument) != type(expected):
        raise AssertionError('%r (%s) != %r (%s)'
                             % (argument, type(argument).__name__,
                                expected, type(expected).__name__))
